/* Main program for Spice 3F5 under Windows95
	Autor: Wolfgang Muees
	Stand: 28.10.97
	Autor: Holger Vogt
	Stand: 01.05.2000
	Stand: 12.12.2001
 $Id: winmain.c,v 1.5 2005/05/24 09:44:22 sjborley Exp $
*/

#include "spice.h"
#include "version.h"
#include "cpstd.h"
#define PACKAGE_STRING "Spice+ "SPICE_VERSION

#define STRICT				// strict type check
#define WIN32_LEAN_AND_MEAN
#include <windows.h>    		// normal Windows calls
#include <windowsx.h>			// Win32 Message Cracker
//#include <ctl3d.h>			// 3D-elements
#include <stdio.h>			// sprintf and co
#include <stdlib.h>			// exit-codes
#include <stdarg.h>			// var. argumente
#include <assert.h>			// assert-macro
// #include <shellapi.h>		// shellexecute
//#include <dir.h>			// Verzeichnis-Funktionen hvogt 09.12.01
//#include <dos.h>			// argc, argv
#include "resource.h"

#include <errno.h>
#include <dirent.h>

#ifdef _MSC_VER
/* Microsoft VC++ specific stuff */
#pragma hdrstop
#endif /* _MSC_VER */

#include <signal.h>
#include <ctype.h>

/* Constants */
#define TBufSize 2048			// Size of the text buffer
#define CR VK_RETURN			// Carriage Return
#define LF 10				// Line Feed
#define SE 0				// String termination
#define BorderSize 8			// Border of the string field
#define SBufSize 100			// Size of the string buffer
#define HistSize 20			// Lines History buffer
#define StatusHeight 25			// Height of the status Bars
#define StatusFrame 2			// Distance status bar / status element
#define StatusElHeight (StatusHeight - 2 * StatusFrame)
#define SourceLength 400		// Place for Source file name
#define AnalyseLength 100		// Place for Analysis

/* Types */
typedef char SBufLine[SBufSize+1];	// Input line

/* Global variables */
HINSTANCE 		hInst;			// Application instance 
int 			WinLineWidth = 640;	// Window width
HWND       	hwMain;				// Main Window of the application 
extern IMPORT HWND      twText;			// Text window 
extern IMPORT HWND	swString;		// Input line
HWND			hwSource;		// Handle of the Source name
extern IMPORT HWND	hwAnalyse;		// Handle of the Analysis Window
static int		nReturnCode	= 0;	// Return value of WinMain
static int		nShowState;		// State of the main window
static WNDCLASSEX hwMainClass;			// Class definition for the main window 
static LPCTSTR	hwClassName  = "SPICE_TEXT_WND";// Class name of the main window 
static LPCTSTR hwWindowName = PACKAGE_STRING;	// main window displayed name 
static WNDCLASSEX twTextClass;			// Class definition for the text box 
static LPCTSTR twClassName  = "SPICE_TEXT_BOX";	// Class name for the text box 
static LPCTSTR twWindowName = "TextOut";	// text box name 
extern IMPORT size_t	TBufEnd;		// Indicates \0
extern IMPORT char TBuffer [TBufSize+1];	// Textbuffer
extern IMPORT SBufLine SBuffer;			// Input buffer
static WNDCLASSEX swStringClass;			// Class definition of the string window
static LPCTSTR swClassName  = "SPICE_STR_IN";	// Class name of the text input
static LPCTSTR swWindowName = "StringIn";	// Name of the window
static char CRLF [] = { CR, LF, SE} ;		// CR/LF
static WNDCLASSEX hwElementClass;			// Class definition of the status announcements
static LPCTSTR hwElementClassName = "ElementClass";
static LPCTSTR hwSourceWindowName = "SourceDisplay";
static LPCTSTR hwAnalyseWindowName = "AnalyseDisplay";
static int RowHeight = 16;		// Height of a text line
static int LineHeight = 25;		// Height of the input line
extern IMPORT int VisibleRows;		// Number of the visible lines in the text window
//static BOOL DoUpdate = FALSE;	        // Text windows update
static WNDPROC swProc = NULL;		// original String window-procedure
static WNDPROC twProc = NULL;		// original Text window-procedure
static SBufLine HistBuffer[HistSize]; 	// History-Buffer for String window
static int HistIndex = 0;		// History-Management
static int HistPtr   = 0;		// History-Management
// Forward-Definition von main()
int xmain( int argc, char * argv[]/*, char * env[]*/);

void SetAnalyse(char * Analyse, int Percent);
void AdjustScroller(void);
void DisplayText(void);
void WaitForIdle(void);

// --------------------------<History-Management>------------------------------

// All buffers extinguish and put pointer on the beginning
void HistoryInit(void)
{
	int i;
	HistIndex = 0;
	HistPtr = 0;
	for ( i = 0; i < HistSize; i++)
		HistBuffer[i][0] = SE;
}

// Extinguish the first line of the buffer; everything moves up
void HistoryScroll(void)
{
	memmove( &(HistBuffer[0]), &(HistBuffer[1]), sizeof(SBufLine) * (HistSize-1));
	HistBuffer[HistSize-1][0] = SE;
	if (HistIndex) HistIndex--;
	if (HistPtr)   HistPtr--;
}

// Write new input line in the History buffer
void HistoryEnter( char * newLine)
{
	if (!newLine || !*newLine) return;
	if (HistPtr == HistSize) HistoryScroll();
	strcpy( HistBuffer[HistPtr], newLine);
	HistPtr++;
	HistIndex = HistPtr;
}

// With the index a line go back and return the entry standing there
char * HistoryGetPrev(void)
{
	if (HistIndex) HistIndex--;
	return &(HistBuffer[HistIndex][0]);
}

// With the index a line go forward and return the entry standing there
char * HistoryGetNext(void)
{
	if (HistIndex < HistPtr) HistIndex++;
	if (HistIndex == HistPtr) HistIndex--;
	return &(HistBuffer[HistIndex][0]);
}


// ---------------------------<Message Handling>-------------------------------

// Vantage point, until no more messages are to be worked on
/*void WaitForIdle(void)
{
	MSG m;
	// arbeite alle Nachrichten ab
	while ( PeekMessage(  &m, NULL, 0, 0, PM_REMOVE)) {
		TranslateMessage( &m);
		DispatchMessage(  &m);
	}
}*/

// -----------------------------<String window>--------------------------------

// Extinguishing of the string window
void ClearInput(void)
{
	// Darstellen
	Edit_SetText( swString, "");
}

// ---------------------------<SourceFile-Window>-----------------------------

// New text in the Sourcefile window
void SetSource( char * Name)
{
	if (hwSource) {
		SetWindowText( hwSource, Name);
		InvalidateRgn( hwSource, NULL, TRUE);
	}
}

// ------------------------------<Analysis-Window>-----------------------------

// New text in the Analysis window
/*static int OldPercent = -2;
void SetAnalyse( char * Analyse, int Percent)
{
	char s[128];

	if (Percent == OldPercent) return;
   OldPercent = Percent;
	if (hwAnalyse) {
		if (Percent < 0)
			sprintf( s, "--ready--");
		else
			sprintf( s, "%s : %3u%%", Analyse, Percent);
		SetWindowText( hwAnalyse, s);
		InvalidateRgn( hwAnalyse, NULL, TRUE);
       WaitForIdle();
	}
}*/

// ------------------------------<Text window>---------------------------------

// Fitting of the Scrollers in the Text window
// Shows at the same time the text anew
/*void AdjustScroller(void)
{
	int LineCount;
	int FirstLine;
	int MyFirstLine;
	LineCount = Edit_GetLineCount( twText);
	FirstLine = Edit_GetFirstVisibleLine( twText);
	MyFirstLine = LineCount - VisibleRows;
	if (MyFirstLine < 0 ) MyFirstLine = 0;
	Edit_Scroll( twText, MyFirstLine - FirstLine, 0);
	// Wars
	DoUpdate = FALSE;
}*/

// Show text
/*void DisplayText( void)
{
	// Explain
	Edit_SetText( twText, TBuffer);
	// Scroller update, new text show
	AdjustScroller();
}*/

// -------------------------------<Window procedures>--------------------------

// Main window changes his Size
#ifdef _MSC_VER
/* Microsoft VC++ specific stuff */
#pragma warn -par
#endif /* _MSC_VER */
void Main_OnSize(HWND hwnd, UINT state, int cx, int cy)
{
	int h = cy - LineHeight - StatusHeight;

	// Text windows expand
	MoveWindow( twText, 0, 0, cx, h , TRUE);
	VisibleRows = (h / RowHeight) -1;
	AdjustScroller();

	// String windows expand
	MoveWindow( swString, 0, h, cx, LineHeight, TRUE);

	// Status elements expand
	h = cy - LineHeight + StatusFrame -1;
	MoveWindow( hwSource, StatusFrame, h, SourceLength, StatusElHeight, TRUE);
	MoveWindow( hwAnalyse, 3 * StatusFrame + SourceLength, h, AnalyseLength,
		StatusElHeight, TRUE);
}

// Write an order in the Spice-command buffer
void PostSpiceCommand( const char * const cmd)
{
	strcpy( SBuffer, cmd);
	strcat( SBuffer, CRLF);
}

// Main window procedure
#ifdef _MSC_VER
/* Microsoft VC++ specific stuff */
#pragma warn -eff
#endif /* _MSC_VER */
LRESULT CALLBACK MainWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
/*	UINT i; */
	
	switch (uMsg) {

	case WM_SYSCOLORCHANGE:
//		Ctl3dColorChange();
		goto DEFAULT_AFTER;

	case WM_CLOSE:
		// write the Spice order for finishing of the program in the text buffer
		PostSpiceCommand( "quit");
		// Interrupt, if simulation already runs, 30.4.2000 hvogt 
		raise (SIGINT);   
		return 0;

/*	// thought for ctrl C, does not go yet
	case WM_KEYDOWN:
		i = (UINT) wParam;
		if ((i == 0x63) && (GetKeyState(VK_CONTROL) < 0)) {
		// Interrupt for interrupting (interactively) 
		// or release finishing (Batch) the program
		   raise (SIGINT);
		   return 0;
		}		*/
		
/*	// thought for ctrl C, does not go yet
	case WM_CHAR:
		i = (char) wParam;
		if ((i == "c") && (GetKeyState(VK_CONTROL) < 0)) {
		// Interrupt for interrupting (interactively) 
		// or release finishing (Batch) the program
		   raise (SIGINT);
		   return 0;
		}   */

	case WM_SIZE:
		HANDLE_WM_SIZE( hwnd, wParam, lParam, Main_OnSize);
		goto DEFAULT_AFTER;

	default:
DEFAULT_AFTER:
		return DefWindowProc( hwnd, uMsg, wParam, lParam);
	}
}


// String window procedure
LRESULT CALLBACK StringWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	char c;
	UINT i;

	switch (uMsg) {

	case WM_KEYDOWN:
		i = (UINT) wParam;
		if ((i == VK_UP) || (i == VK_DOWN)) {
			// put old text
			SetWindowText( hwnd, i == VK_UP? HistoryGetPrev(): HistoryGetNext());
			// Cursor at the end of the line
			CallWindowProc( swProc, hwnd, uMsg, (WPARAM) VK_END, lParam);
			return 0;
		}
		if ( i == VK_ESCAPE) {
			ClearInput();
			return 0;
		}


		goto DEFAULT;

	case WM_CHAR:
			c = (char) wParam;
			if (c == CR) {
				GetWindowText( hwnd, SBuffer, SBufSize);
				HistoryEnter( SBuffer);
				strcat( SBuffer, CRLF);
				ClearInput();
				return 0;
			}
			if (c == VK_ESCAPE)
				return 0;
	default:
DEFAULT:
		return CallWindowProc( swProc, hwnd, uMsg, wParam, lParam);
	}
}


// Text window procedure
LRESULT CALLBACK TextWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	unsigned char c;
	UINT i;

	switch (uMsg) {

	case WM_KEYDOWN:
		i = (UINT) wParam;
		if ((i == VK_UP) || (i == VK_DOWN) || (i == VK_ESCAPE)) {
			// Lead around in the string window
			SetFocus( swString);
			return SendMessage( swString, uMsg, wParam, lParam);
		}
		goto DEFAULT_TEXT;

	case WM_CHAR:
		c = (unsigned char) wParam;
		if ((c == CR) || ( c >= ' ') || ( c == VK_ESCAPE)) {
			// Lead around in the string window
			SetFocus( swString);
			return SendMessage( swString, uMsg, wParam, lParam);
		}
	default:
DEFAULT_TEXT:
		return CallWindowProc( twProc, hwnd, uMsg, wParam, lParam);
	}
}


void Element_OnPaint(HWND hwnd)
{
	PAINTSTRUCT ps;
	RECT r;
	RECT s;
	HGDIOBJ o;
	char buffer[128];
	int i;

	// Prepare
	HDC hdc = BeginPaint( hwnd, &ps);
	GetClientRect( hwnd, &r);

	// Frame sign
	o = GetStockObject( GRAY_BRUSH);
	s.left  	= r.left;
	s.right		= r.right;
	s.top		= r.top;
	s.bottom	= r.top+1;
	FillRect( hdc, &s, o);

	s.right     = r.left+1;
	s.bottom	= r.bottom;
	FillRect( hdc, &s, o);

	o = GetStockObject( WHITE_BRUSH);
	s.right		= r.right;
	s.top		= r.bottom-1;
	FillRect( hdc, &s, o);

	s.left		= r.right-1;
	s.top		= r.top;
	FillRect( hdc, &s, o);

	// Draw contents
	buffer[0] = '\0';
	i = GetWindowText( hwnd, buffer, 127);
	s.left 		= r.left+1;
	s.right		= r.right-1;
	s.top		= r.top+1;
	s.bottom	= r.bottom-1;
	o = GetStockObject( LTGRAY_BRUSH);
	FillRect( hdc, &s, o);
	SetBkMode( hdc, TRANSPARENT);
	ExtTextOut( hdc, s.left+1, s.top+1, ETO_CLIPPED, &s, buffer, i, NULL);

	// Ende
	EndPaint( hwnd, &ps);
}


// Element window procedure
LRESULT CALLBACK ElementWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_PAINT:
		HANDLE_WM_PAINT(hwnd, wParam, lParam, Element_OnPaint);
		return 0;

	default:
		return DefWindowProc( hwnd, uMsg, wParam, lParam);
	}
}


#define SPACE			' '
#define QUOTE			'\"'
#define DELIMITER		26		/* for the lack of anything better */
#define DELIMITERSTRING	"\26"

/*
	This function converts a string into an argc/argv represenation.
	INPUT:
		cmdline		-	a string
	OUTPUT:
		argc		-	the number of equivalent argv strings
					which is also the number of strings in argv
		argv		-	the argv given the input string which
					consists of seperate strings for each argument
	RETURNS:
		0  on success
		-1 on failure
*/
int MakeArgcArgv(char *cmdline,int *argc,char ***argv)
{
	char  *pC1;			/*	a temporary character pointer */
	char  *pC2;			/*	a temporary character pointer */
	char  *pWorkString=NULL;		/*	a working copy of cmdline */
	int    i;				/*	a loop counter */
	int    j;				/*	a loop counter */
	int    quoteflag=0;			/*	for the finite state machine parsing cmdline */
	int    numargs=1;			/*	the number of command line arguments, later
						copied to *argc */
	char **tmpargv;			/*	the temporary argv, later copied to *argv */
	int    status = ERROR_SUCCESS; 	/* status */
	char   buffer[MAX_PATH+1];


	/* make sure we aren't dealing with any NULL pointers */
	if (	(NULL == argc)
	|| (NULL == argv))
	{
		status = -1;
		goto outahere;
	}
	*argc = 0;		/* set the count to zero to start */
	*argv = NULL;	/* set the pointer to NULL to start */
	/*	if the string passed in was a NULL pointer, consider this
		to be an empty command line and give back only
		an argc of 1 and an argv[0] */
	if (NULL != cmdline)
	{
	/*	make a copy of the string so that we can modify it
			without messing up the original */
		pWorkString = strdup(cmdline);
		if (NULL == pWorkString)
			return -1; /* memory allocation error */
		/*	Now, to make sure we don't have any quoted arguments
			with spaces in them, replace all spaces except those
			between " marks with our own special delimiter for
			strtok */
		/* trim all the whitespace off the end of the string. */
		for (i=(signed)strlen(pWorkString)-1; i >=0; i--)
			if (isspace(pWorkString[i]))
				pWorkString[i] = '\0';
			else
				break;
		/*	If we still have a string left, parse it for all
			the arguments. */
		if (strlen(pWorkString))
		{
			/*	This could probably be done with strtok as well
				but strtok is destructive if I wanted to look for " \""
				and I couldn't tell what delimiter that I had bumped
				against */
			for (i=0; i < (signed)strlen(pWorkString); i++)
			{
				switch (pWorkString[i])
				{
				case SPACE:
					if (!quoteflag)
					{
						pWorkString[i] = DELIMITER;  /* change space to delimiter */
						numargs++;
					}
					break;
				case QUOTE:
					quoteflag = !quoteflag; /* turns on and off as we pass quotes */
					break;
				}
			}
			/*	Now, we should have ctrl-Zs everywhere that
				there used to be a space not protected by
				quote marks.  We should also have the number
				of command line arguments that were in the
				command line (not including argv[0] which should
				be the program name).  We should add one more
				to numargs to take into account argv[0].  */
			numargs++;
		}
	}
	/* malloc an argv */
	tmpargv = (char**)malloc(numargs * sizeof(char *));
	if (NULL == tmpargv)
	{
		status = -1;
		goto outahere;
	}
	/*	you can put your program name here or find an API to give it
		to you if you want.  I am just giving a name to fill the space */
	GetModuleFileName(NULL, buffer, sizeof(buffer));


//	tmpargv[0] = strdup("ngspice");
	tmpargv[0] = buffer;
	
	pC1 = NULL;
	/*	Now actually strdup all the arguments out of the sting
		and store them in the argv */
	for (i=1; i < numargs; i++)
	{
		if (NULL == pC1)
			pC1 = pWorkString;
		for (j=0; j < (signed)strlen(pC1); j++)
		{
			if (DELIMITER == pC1[j])
			{
				pC1[j] = '\0';
				pC2    = &pC1[j+1];
				break;
			}
		}
		tmpargv[i] = strdup(pC1);
		if (NULL == tmpargv[i])
		{
			status = -1;
			goto outahere;
		}
		pC1 = pC2;
	}
	/*	copy the working values over to the arguments */
	*argc = numargs;
	*argv = tmpargv;
outahere:
	/*	free the working string if one was allocated */
	if (pWorkString)
		free(pWorkString);
	/* return status */
	return status;
}

/* Main entry point for our Windows application */
#ifdef _MSC_VER
/* Microsoft VC++ specific stuff */
#pragma warn -par
#endif /* _MSC_VER */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
	int i;
	int status;
	
	int argc;
	char **argv;

	// global variables fill
	hInst = hInstance;
	nShowState = nCmdShow;

	// Text buffers initialize
	TBufEnd = 0;
	TBuffer[TBufEnd] = SE;
	SBuffer[0] = SE;
	HistoryInit();

	// 3D-Elements registration
//	Ctl3dRegister( hInstance);
//	Ctl3dAutoSubclass( hInstance);

	// Define Main windows class
        hwMainClass.cbSize = sizeof(WNDCLASSEX);
	hwMainClass.style 			= CS_HREDRAW | CS_VREDRAW;
	hwMainClass.lpfnWndProc		= MainWindowProc;
	hwMainClass.cbClsExtra		= 0;
	hwMainClass.cbWndExtra		= 0;
	hwMainClass.hInstance		= hInst;
	hwMainClass.hIcon 			= LoadIcon( hInst, MAKEINTRESOURCE(IDI_ICON1));
        hwMainClass.hIconSm = hwMainClass.hIcon;
	hwMainClass.hCursor			= LoadCursor( NULL, IDC_ARROW);
	hwMainClass.hbrBackground 	= GetStockObject( LTGRAY_BRUSH);
	hwMainClass.lpszMenuName 	= NULL;
	hwMainClass.lpszClassName 	= hwClassName;
	if (!RegisterClassEx( &hwMainClass)) goto THE_END;

	// Define Text class
	if (!GetClassInfoEx( NULL, "EDIT", &twTextClass)) goto THE_END;
	twProc = twTextClass.lpfnWndProc;
        twTextClass.cbSize = sizeof(WNDCLASSEX);
	twTextClass.lpfnWndProc		= TextWindowProc;
	twTextClass.hInstance		= hInst;
	twTextClass.lpszMenuName 	= NULL;
	twTextClass.lpszClassName 	= twClassName;
	if (!RegisterClassEx( &twTextClass)) goto THE_END;

	// Define String class
	if (!GetClassInfoEx( NULL, "EDIT", &swStringClass)) goto THE_END;
	swProc = swStringClass.lpfnWndProc;
        swStringClass.cbSize = sizeof(WNDCLASSEX);
	swStringClass.lpfnWndProc   = StringWindowProc;
	swStringClass.hInstance		= hInst;
	swStringClass.lpszMenuName 	= NULL;
	swStringClass.lpszClassName = swClassName;
	if (!RegisterClassEx( &swStringClass)) goto THE_END;

	// Define Status Element class
        hwElementClass.cbSize = sizeof(WNDCLASSEX);
	hwElementClass.style 			= CS_HREDRAW | CS_VREDRAW;
	hwElementClass.lpfnWndProc		= ElementWindowProc;
	hwElementClass.cbClsExtra		= 0;
	hwElementClass.cbWndExtra		= 0;
	hwElementClass.hInstance		= hInst;
	hwElementClass.hIcon 			= NULL;
	hwElementClass.hCursor			= LoadCursor( NULL, IDC_ARROW);
	hwElementClass.hbrBackground 	= GetStockObject( LTGRAY_BRUSH);
	hwElementClass.lpszMenuName 	= NULL;
	hwElementClass.lpszClassName 	= hwElementClassName;
	if (!RegisterClassEx( &hwElementClass)) goto THE_END;

	// Create the Main window
	i = GetSystemMetrics( SM_CYSCREEN) / 3;
	hwMain = CreateWindow( hwClassName, hwWindowName, WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
		0, i * 2, GetSystemMetrics( SM_CXSCREEN), i, NULL, NULL, hInst, NULL);
	if (!hwMain) goto THE_END;

	// Create Text window
	twText = CreateWindowEx(WS_EX_NOPARENTNOTIFY, twClassName, twWindowName,
		ES_LEFT | ES_MULTILINE | ES_READONLY | WS_CHILD | WS_BORDER | WS_VSCROLL,
		20,20,300,100, hwMain, NULL, hInst, NULL);
	if (!twText) goto THE_END;
	// Ansii fixed font
	{
		HDC textDC;
		HFONT font;
		TEXTMETRIC tm;
		font = GetStockFont( ANSI_FIXED_FONT);
		SetWindowFont( twText, font, FALSE);
		textDC = GetDC( twText);
		if (textDC) {
			SelectObject( textDC, font);
			if (GetTextMetrics( textDC, &tm)) {
				RowHeight = tm.tmHeight;
				WinLineWidth = 90 * tm.tmAveCharWidth;
			}
			ReleaseDC( twText, textDC);
		}
	}

	// Create String window
	swString = CreateWindowEx(WS_EX_NOPARENTNOTIFY, swClassName, swWindowName,
		ES_LEFT | WS_CHILD | WS_BORDER, 20,20,300,100, hwMain, NULL, hInst, NULL);
	if (!swString) goto THE_END;
	{
		HDC stringDC;
		TEXTMETRIC tm;
		stringDC = GetDC( swString);
		if (stringDC) {
			if (GetTextMetrics( stringDC, &tm))
				LineHeight = tm.tmHeight + tm.tmExternalLeading + BorderSize;
			ReleaseDC( swString, stringDC);
		}
	}

	// Create Source window
	hwSource = CreateWindowEx(WS_EX_NOPARENTNOTIFY, hwElementClassName,
		hwSourceWindowName, WS_CHILD, 0,0, SourceLength, StatusElHeight, hwMain,
		NULL, hInst, NULL);
	if (!hwSource) goto THE_END;


	// Create Analysis window
	hwAnalyse = CreateWindowEx(WS_EX_NOPARENTNOTIFY, hwElementClassName,
		hwAnalyseWindowName, WS_CHILD, 0,0, AnalyseLength, StatusElHeight, hwMain,
		NULL, hInst, NULL);
	if (!hwAnalyse) goto THE_END;


	// Show main window with the subwindows
	if (WinLineWidth > 600) WinLineWidth = 600;
	MoveWindow( hwMain, 0, ((GetSystemMetrics( SM_CYSCREEN) / 3) * 2 - 22), WinLineWidth,
				GetSystemMetrics( SM_CYSCREEN) / 3, FALSE);
	ShowWindow( hwMain,   nShowState);
	ShowWindow( twText,   SW_SHOWNORMAL);
	ShowWindow( swString, SW_SHOWNORMAL);
	ShowWindow( hwSource, SW_SHOWNORMAL);
	ShowWindow( hwAnalyse,SW_SHOWNORMAL);
	ClearInput();
	DisplayText();
	SetSource( "");
//   SetAnalyse( NULL, -1);
        SetAnalyse(" ", 0);
	UpdateWindow( hwMain);
	SetFocus( swString);

        status = MakeArgcArgv(lpszCmdLine,&argc,&argv);


	// Wait, until everything is clear
	WaitForIdle();

	// After call xmain()
	nReturnCode = xmain(argc, argv/*, _environ*/);
	

THE_END:
	// 3D switch off
//	Ctl3dUnregister( hInstance);

	// terminate
	return nReturnCode;
}

