/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1985 Thomas L. Quarles
**********/

#include "spice.h"
#include <stdio.h>

#include "dlopen.h"
#include "cktdefs.h"
#include "acdefs.h"
#include "devdefs.h"
#include "util.h"
#include "sperror.h"
#include "suffix.h"

#ifdef XSPICE
/* gtri - add - wbk - 12/19/90 - Add headers */ 
#include "mif.h"
#include "evtproto.h"
#include "ipctiein.h"
/* gtri - end - wbk */

extern IMPORT void *xspicelib;
#endif

EXPORT int
ACan(CKTcircuit *ckt, int restart) 
{
    double freq;
    double freqTol; /* tolerence parameter for finding final frequency */
    int error;
    #ifdef HAS_SENSE2
    long save;
    int save1;
    #endif
    int numNames;
    IFuid *nameList;
    IFuid freqUid;
    static GENERIC *acPlot;
    GENERIC *plot;

/* gtri - add - wbk - 12/19/90 - Add IPC stuff and anal_init and anal_type */
#ifdef XSPICE
    #ifndef DYNAMIC
    /* Tell the beginPlot routine what mode we're in */
    g_ipc.anal_type = IPC_ANAL_AC;

    /* Tell the code models what mode we're in */
    g_mif_info.circuit.anal_type = MIF_DC;

    g_mif_info.circuit.anal_init = MIF_TRUE;
    #else
    //Ipc_Tiein_t *pg_ipc;
    Mif_Info_t *pg_mif_info = NULL;
    if (xspicelib)
        {
        //pg_ipc = (Ipc_Tiein_t *) DLSYM(xspicelib, "g_ipc");
        pg_mif_info = (Mif_Info_t *) DLSYM(xspicelib, "g_mif_info");

        /* Tell the beginPlot routine what mode we're in */
        //pg_ipc->anal_type = IPC_ANAL_AC;

        /* Tell the code models what mode we're in */
        pg_mif_info->circuit.anal_type = MIF_DC;

        pg_mif_info->circuit.anal_init = MIF_TRUE;
        }
    #endif
#endif
/* gtri - end - wbk */

    if(((ACAN*)ckt->CKTcurJob)->ACsaveFreq == 0 || restart) {
        /* start at beginning */

	if (((ACAN*)ckt->CKTcurJob)->ACnumberSteps < 1)
	    ((ACAN*)ckt->CKTcurJob)->ACnumberSteps = 1;

        switch(((ACAN*)ckt->CKTcurJob)->ACstepType) {

        case DECADE:
            ((ACAN*)ckt->CKTcurJob)->ACfreqDelta =
                    exp(log(10.0)/((ACAN*)ckt->CKTcurJob)->ACnumberSteps);
            break;
        case OCTAVE:
            ((ACAN*)ckt->CKTcurJob)->ACfreqDelta =
                    exp(log(2.0)/((ACAN*)ckt->CKTcurJob)->ACnumberSteps);
            break;
        case LINEAR:
	    if (((ACAN*)ckt->CKTcurJob)->ACnumberSteps-1 > 1)
		((ACAN*)ckt->CKTcurJob)->ACfreqDelta =
                    (((ACAN*)ckt->CKTcurJob)->ACstopFreq -
                    ((ACAN*)ckt->CKTcurJob)->ACstartFreq)/
                    (((ACAN*)ckt->CKTcurJob)->ACnumberSteps-1);
	    else
		((ACAN*)ckt->CKTcurJob)->ACfreqDelta = HUGE;
            break;
        default:
            return(E_BADPARM);
        }
#ifdef XSPICE
        if(ckt->evt->counts.num_insts == 0) {
            /* If no event-driven instances, do what SPICE normally does */
            error = CKTop(ckt,
                (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITJCT,
                (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITFLOAT,
                ckt->CKTdcMaxIter);
            if(error) return(error);
        }
        else {
            #ifndef DYNAMIC
            /* Else, use new DCOP algorithm */
            error = EVTop(ckt,
                        (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITJCT,
                        (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITFLOAT,
                        ckt->CKTdcMaxIter,
                        MIF_TRUE);
            EVTdump(ckt, IPC_ANAL_DCOP, 0.0);
            EVTop_save(ckt, MIF_TRUE, 0.0);
            if(error)
                return(error);
            #else
            if (xspicelib)
                {
                int (*evtop)(CKTcircuit *, long, long, int, Mif_Boolean_t);
                void (*evtdump)(CKTcircuit *, Ipc_Anal_t, double);
                void (*evtopsave)(CKTcircuit *, Mif_Boolean_t, double);
                evtop = (int (*)(CKTcircuit *, long, long, int, Mif_Boolean_t))
                        DLSYM(xspicelib, PREFIX"EVTop");
                evtdump = (void (*)(CKTcircuit *, Ipc_Anal_t, double))
                          DLSYM(xspicelib, PREFIX"EVTdump");
                evtopsave = (void (*)(CKTcircuit *, Mif_Boolean_t, double))
                            DLSYM(xspicelib, PREFIX"EVTop_save");

                /* Else, use new DCOP algorithm */
                error = evtop(ckt,
                            (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITJCT,
                            (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITFLOAT,
                            ckt->CKTdcMaxIter,
                            MIF_TRUE);
                evtdump(ckt, IPC_ANAL_DCOP, 0.0);
                evtopsave(ckt, MIF_TRUE, 0.0);
                if(error)
                    return(error);
                }
            #endif
        }
/* gtri - end - wbk - Call EVTop if event-driven instances exist */
#else	
        error = CKTop(ckt,
                (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITJCT,
                (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITFLOAT,
                ckt->CKTdcMaxIter);
        if(error) return(error);
#endif

#if (defined XSPICE) && (defined linux)
/* gtri - add - wbk - 12/19/90 - Add IPC stuff */

        /* Send the operating point results for Mspice compatibility */
        if(g_ipc.enabled)
		{
            /* Call CKTnames to get names of nodes/branches used by BeginPlot */
            /* Probably should free nameList after this block since called again... */
            error = CKTnames(ckt,&numNames,&nameList);
            if(error) return(error);

            /* We have to do a beginPlot here since the data to return is */
            /* different for the DCOP than it is for the AC analysis.  Moreover */
            /* the begin plot has not even been done yet at this point... */
            (*(SPfrontEnd->OUTpBeginPlot))((void *)ckt,(void*)ckt->CKTcurJob,
                ckt->CKTcurJob->JOBname,(IFuid)NULL,IF_REAL,numNames,nameList,
                IF_REAL,&acPlot);
	    tfree(nameList);

            ipc_send_dcop_prefix();
            CKTdump(ckt,(double)0,acPlot);
            ipc_send_dcop_suffix();

            (*(SPfrontEnd->OUTendPlot))(acPlot);
        }

/* gtri - end - wbk */
#endif

        ckt->CKTmode = (ckt->CKTmode & MODEUIC) | MODEDCOP | MODEINITSMSIG;
        error = CKTload(ckt);
        if(error) return(error);

        error = CKTnames(ckt,&numNames,&nameList);
        if(error) return(error);

	if (ckt->CKTkeepOpInfo) {
	    /* Dump operating point. */
	    error = (*(SPfrontEnd->OUTpBeginPlot))((GENERIC *)ckt,
		(GENERIC*)ckt->CKTcurJob, "AC Operating Point",
		(IFuid)NULL,IF_REAL,numNames,nameList, IF_REAL,&plot);
	    if(error) return(error);
	    CKTdump(ckt,(double)0,plot);
	    (*(SPfrontEnd->OUTendPlot))(plot);
	}

        (*(SPfrontEnd->IFnewUid))((GENERIC *)ckt,&freqUid,(IFuid)NULL,
                "frequency", UID_OTHER,(GENERIC **)NULL);
        error = (*(SPfrontEnd->OUTpBeginPlot))((GENERIC *)ckt,
		(GENERIC*)ckt->CKTcurJob,
                ckt->CKTcurJob->JOBname,freqUid,IF_REAL,numNames,nameList,
                IF_COMPLEX,&acPlot);
	if(error) return(error);

        if (((ACAN*)ckt->CKTcurJob)->ACstepType != LINEAR) {
	    (*(SPfrontEnd->OUTattributes))((GENERIC *)acPlot,NULL,
		    OUT_SCALE_LOG, NULL);
	}
        freq = ((ACAN*)ckt->CKTcurJob)->ACstartFreq;

    } else {    /* continue previous analysis */
        freq = ((ACAN*)ckt->CKTcurJob)->ACsaveFreq;
        ((ACAN*)ckt->CKTcurJob)->ACsaveFreq = 0; /* clear the 'old' frequency */
    }
    switch(((ACAN*)ckt->CKTcurJob)->ACstepType) {
    case DECADE:
    case OCTAVE:
        freqTol = ((ACAN*)ckt->CKTcurJob)->ACfreqDelta * 
                ((ACAN*)ckt->CKTcurJob)->ACstopFreq * ckt->CKTreltol;
        break;
    case LINEAR:
        freqTol = ((ACAN*)ckt->CKTcurJob)->ACfreqDelta * ckt->CKTreltol;
        break;
    default:
        return(E_BADPARM);
    }

/* gtri - add - wbk - 12/19/90 - Set anal_init and anal_type */
#ifdef XSPICE
    #ifndef DYNAMIC
    g_mif_info.circuit.anal_init = MIF_TRUE;

    /* Tell the code models what mode we're in */
    g_mif_info.circuit.anal_type = MIF_AC;
    #else
    if (xspicelib)
        {
        pg_mif_info->circuit.anal_init = MIF_TRUE;

        /* Tell the code models what mode we're in */
        pg_mif_info->circuit.anal_type = MIF_AC;
        }
    #endif
#endif
/* gtri - end - wbk */

    while(freq <= ((ACAN*)ckt->CKTcurJob)->ACstopFreq+freqTol) {

        if( (*(SPfrontEnd->IFpauseTest))() ) {
            /* user asked us to pause via an interrupt */
            ((ACAN*)ckt->CKTcurJob)->ACsaveFreq = freq;
            return(E_PAUSE);
        }
        ckt->CKTomega = 2.0 * M_PI *freq;
        ckt->CKTmode = (ckt->CKTmode&MODEUIC) | MODEAC;

        error = NIacIter(ckt);
        if(error) return(error);


#ifdef HAS_SENSE2
        if(ckt->CKTsenInfo && (ckt->CKTsenInfo->SENmode&ACSEN) ){

            save = ckt->CKTmode;
            ckt->CKTmode=(ckt->CKTmode&MODEUIC)|MODEDCOP|MODEINITSMSIG;
            save1 = ckt->CKTsenInfo->SENmode;
            ckt->CKTsenInfo->SENmode = ACSEN;
            if(freq == ((ACAN*)ckt->CKTcurJob)->ACstartFreq){
                ckt->CKTsenInfo->SENacpertflag = 1;
            }
            else{
                ckt->CKTsenInfo->SENacpertflag = 0;
            }
            if(error = CKTsenAC(ckt)) return (error);
            ckt->CKTmode = save;
            ckt->CKTsenInfo->SENmode = save1;
        }
#endif
/* gtri - modify - wbk - 12/19/90 - Send IPC stuff */
#if (defined XSPICE) && (defined linux)
        if(g_ipc.enabled)
            ipc_send_data_prefix(freq);

        error = CKTacDump(ckt,freq,acPlot);

        if(g_ipc.enabled)
            ipc_send_data_suffix();

/* gtri - modify - wbk - 12/19/90 - Send IPC stuff */
#else
        error = CKTacDump(ckt,freq,acPlot);
#endif
        if(error) return(error);

        /*  increment frequency */

        switch(((ACAN*)ckt->CKTcurJob)->ACstepType) {
        case DECADE:
        case OCTAVE:
            freq *= ((ACAN*)ckt->CKTcurJob)->ACfreqDelta;
            if(((ACAN*)ckt->CKTcurJob)->ACfreqDelta==1) goto endsweep;
            break;
        case LINEAR:
            freq += ((ACAN*)ckt->CKTcurJob)->ACfreqDelta;
            if(((ACAN*)ckt->CKTcurJob)->ACfreqDelta==0) goto endsweep;
            break;
        default:
            return(E_INTERN);
        }
    }
endsweep:
    (*(SPfrontEnd->OUTendPlot))(acPlot);
    return(0);
}

