/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1985 Thomas L. Quarles
**********/

#include "spice.h"
#include <stdio.h>

#include "dlopen.h"
#include "cktdefs.h"
#include "acdefs.h"
#include "devdefs.h"
#include "util.h"
#include "sperror.h"
#include "spicedev.h"
#include "suffix.h"

#ifdef XSPICE
/* gtri - add - wbk - 12/19/90 - Add headers */ 
#include "mif.h"
#include "evtproto.h"
#include "ipctiein.h"
/* gtri - end - wbk */

extern IMPORT void *xspicelib;
#endif

    /* CKTacLoad(ckt)
     * this is a driver program to iterate through all the various
     * ac load functions provided for the circuit elements in the
     * given circuit 
     */


EXPORT int CKTacLoad(register CKTcircuit *ckt)
{
    register int i;
    register int size;
    int error;
    size = SMPmatSize(ckt->CKTmatrix);
    for (i=0;i<=size;i++) {
        *(ckt->CKTrhs+i)=0;
        *(ckt->CKTirhs+i)=0;
    }
    SMPcClear(ckt->CKTmatrix);

    for (i=0;i<DEVmaxnum;i++) {
        if (DEVices[i] != NULL)
        if ( ((*DEVices[i]).DEVacLoad != NULL) && (ckt->CKThead[i] != NULL) ){
            error = (*((*DEVices[i]).DEVacLoad))(ckt->CKThead[i],ckt);
            if(error) return(error);
        }
    }

#ifdef XSPICE
   /* gtri - begin - Put resistors to ground at all nodes. */
    /* Value of resistor is set by new "rshunt" option.     */

    if(ckt->enh->rshunt_data.enabled) {
       for(i = 0; i < ckt->enh->rshunt_data.num_nodes; i++) {
          *(ckt->enh->rshunt_data.diag[i]) +=
                               ckt->enh->rshunt_data.gshunt;
       }
    }

    /* gtri - end - Put resistors to ground at all nodes */



    /* gtri - add - wbk - 11/26/90 - reset the MIF init flags */
    #ifndef DYNAMIC
    /* init is set by CKTinit and should be true only for first load call */
    g_mif_info.circuit.init = MIF_FALSE;

    /* anal_init is set by CKTdoJob and is true for first call */
    /* of a particular analysis type */
    g_mif_info.circuit.anal_init = MIF_FALSE;
    #else
    if (xspicelib)
        {
        Mif_Info_t *pg_mif_info;
        pg_mif_info = (Mif_Info_t *) DLSYM(xspicelib, "g_mif_info");

        /* init is set by CKTinit and should be true only for first load call */
        pg_mif_info->circuit.init = MIF_FALSE;

        /* anal_init is set by CKTdoJob and is true for first call */
        /* of a particular analysis type */
        pg_mif_info->circuit.anal_init = MIF_FALSE;
        }
    #endif
    /* gtri - end - wbk - 11/26/90 */
#endif

    return(OK);
}
