/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1985 Thomas L. Quarles
**********/
/*
 */

    /* CKTload(ckt)
     * this is a driver program to iterate through all the various
     * load functions provided for the circuit elements in the
     * given circuit 
     */

#include "spice.h"
#include <stdio.h>

#include "dlopen.h"
#include "smpdefs.h"
#include "cktdefs.h"
#include "devdefs.h"
#include "sperror.h"
#include "spicedev.h"
#include "suffix.h"

#ifdef XSPICE
/* gtri - add - wbk - 11/26/90 - add include for MIF global data */
#include "mif.h"
/* gtri - end - wbk - 11/26/90 */

extern IMPORT void *xspicelib;
#endif

static int ZeroNoncurRow( );

EXPORT int CKTload(register CKTcircuit *ckt)
{
    register int i;
    register int size;
    double startTime;
    CKTnode *node;
    int error;
#ifdef STEPDEBUG
    int noncon;
#endif /* STEPDEBUG */

#ifdef XSPICE
  /* gtri - begin - Put resistors to ground at all nodes */
    /*   SMPmatrix  *matrix; maschmann : deleted , because unused */

    double gshunt;
    int num_nodes;

    /* gtri - begin - Put resistors to ground at all nodes */
#endif

    startTime = (*(SPfrontEnd->IFseconds))();
    size = SMPmatSize(ckt->CKTmatrix);
    for (i=0;i<=size;i++) {
        *(ckt->CKTrhs+i)=0;
    }
    SMPclear(ckt->CKTmatrix);
#ifdef STEPDEBUG
    noncon = ckt->CKTnoncon;
#endif /* STEPDEBUG */

    for (i=0;i<DEVmaxnum;i++) {
        if (DEVices[i] != NULL)
        if ( ((*DEVices[i]).DEVload != NULL) && (ckt->CKThead[i] != NULL) ){
            error = (*((*DEVices[i]).DEVload))(ckt->CKThead[i],ckt);
	    if (ckt->CKTnoncon)
		ckt->CKTtroubleNode = 0;
#ifdef STEPDEBUG
            if(noncon != ckt->CKTnoncon) {
                printf("device type %s nonconvergence\n",
                        (*DEVices[i]).DEVpublic.name);
                noncon = ckt->CKTnoncon;
            }
#endif /* STEPDEBUG */
            if(error) return(error);
        }
    }

#ifdef XSPICE
    /* gtri - add - wbk - 11/26/90 - reset the MIF init flags */
    #ifndef DYNAMIC
    /* init is set by CKTinit and should be true only for first load call */
    g_mif_info.circuit.init = MIF_FALSE;

    /* anal_init is set by CKTdoJob and is true for first call */
    /* of a particular analysis type */
    g_mif_info.circuit.anal_init = MIF_FALSE;
    #else
    if (xspicelib)
        {
        Mif_Info_t *pg_mif_info;
        pg_mif_info = (Mif_Info_t *) DLSYM(xspicelib, "g_mif_info");

        /* init is set by CKTinit and should be true only for first load call */
        pg_mif_info->circuit.init = MIF_FALSE;

        /* anal_init is set by CKTdoJob and is true for first call */
        /* of a particular analysis type */
        pg_mif_info->circuit.anal_init = MIF_FALSE;
        }
    #endif
    /* gtri - end - wbk - 11/26/90 */

    /* gtri - begin - Put resistors to ground at all nodes. */
    /* Value of resistor is set by new "rshunt" option.     */

    if(ckt->enh->rshunt_data.enabled) {
       gshunt = ckt->enh->rshunt_data.gshunt;
       num_nodes = ckt->enh->rshunt_data.num_nodes;
       for(i = 0; i < num_nodes; i++) {
          *(ckt->enh->rshunt_data.diag[i]) += gshunt;
       }
    }
#endif
    /* gtri - end - Put resistors to ground at all nodes */

    if(ckt->CKTmode & MODEDC) {
        /* consider doing nodeset & ic assignments */
        if(ckt->CKTmode & (MODEINITJCT | MODEINITFIX)) {
            /* do nodesets */
            for(node=ckt->CKTnodes;node;node=node->next) {
                if(node->nsGiven) {
		    if (ZeroNoncurRow(ckt->CKTmatrix, ckt->CKTnodes,
			node->number))
		    {
			*(ckt->CKTrhs+node->number) = 1.0e10 * node->nodeset;
			*(node->ptr) = 1e10;
		    } else {
			*(ckt->CKTrhs+node->number) = node->nodeset;
			*(node->ptr) = 1;
		    }
		}
            }
        }
        if( (ckt->CKTmode & MODETRANOP) && (!(ckt->CKTmode & MODEUIC))) {
            for(node=ckt->CKTnodes;node;node=node->next) {
                if(node->icGiven) {
		    if (ZeroNoncurRow(ckt->CKTmatrix, ckt->CKTnodes,
			node->number))
		    {
			*(ckt->CKTrhs+node->number) = 1.0e10 * node->ic;
			*(node->ptr) = 1e10;
		    } else {
			*(ckt->CKTrhs+node->number) = node->ic;
			*(node->ptr) = 1;
		    }
                }
            }
        }
    }
    ckt->CKTstat->STATloadTime += (*(SPfrontEnd->IFseconds))()-startTime;
    /* SMPprint(ckt->CKTmatrix, stdout); if you want to debug, this is a
	good place to start ... */
    return(OK);
}

static int
ZeroNoncurRow(matrix, nodes, rownum)
	SMPmatrix	*matrix;
	CKTnode		*nodes;
	int		rownum;
{
	CKTnode		*n;
	double		*x;
	int		currents;

	currents = 0;
	for (n = nodes; n; n = n->next) {
	    x = (double *) SMPfindElt(matrix, rownum, n->number, 0);
	    if (x) {
		if (n->type == SP_CURRENT)
		    currents = 1;
		else
		    *x = 0.0;
	    }
	}
	return currents;
}
