
#include "spice.h"
#include "cpdefs.h"
#include "ftedefs.h"
#include "util.h"

#include "dlopen.h"
#ifdef XSPICE
#include "mif.h"
#include "devdefs.h"
#include "spicedev.h"
#include "dllitf.h" // the coreInfo Structure
#include "evtudn.h" // Use defined nodes

extern IMPORT int *DEVicesfl;
extern IMPORT void *xspicelib;
extern IMPORT int xspice_loaded;
#ifndef DYNAMIC
extern struct coreInfo_t  coreInfo;
#endif

IFdevice **
devices_ptr(void)
{
    return (IFdevice **) DEVices;
}

static void relink() {
  ft_sim->numDevices = DEVmaxnum;
  ft_sim->devices = devices_ptr();

  return;
}

int add_device(int n, SPICEdev **devs, int flag){
  int i;

  DEVices = (SPICEdev **)trealloc(DEVices,(DEVmaxnum+n)*sizeof(SPICEdev *));
  DEVicesfl = (int *)trealloc(DEVicesfl,(DEVmaxnum+n)*sizeof(int));

  #ifdef DYNAMIC
  int *mifisize;
  mifisize = (int *) DLSYM(xspicelib, "MIFiSize");
  #endif
  for(i = 0; i < n;i++){
    printf("Added device: %s\n",devs[i]->DEVpublic.name); // debug
    DEVices[DEVmaxnum+i] = devs[i];

    // added by SDB on 6.20.2003
    #ifndef DYNAMIC
    DEVices[DEVmaxnum+i]->DEVinstSize = &MIFiSize;
    #else
    DEVices[DEVmaxnum+i]->DEVinstSize = mifisize;
    #endif

    DEVicesfl[DEVmaxnum+i] = flag;
  }
  DEVmaxnum += n;
  relink();
  return 0;
}

int add_udn(int n,Evt_Udn_Info_t **udns){
  int i;
  g_evt_udn_info = (Evt_Udn_Info_t  **)trealloc(g_evt_udn_info,(g_evt_num_udn_types+n)*sizeof(Evt_Udn_Info_t  *));
  for(i = 0; i < n;i++){
    /*debug*/printf("Added udn: %s\n",udns[i]->name);
    g_evt_udn_info[g_evt_num_udn_types+i] = udns[i];
  }
  g_evt_num_udn_types += n;
  return 0;
}

int load_opus(char *name){
  void *lib;
  const char *msg = NULL;
  int *num = NULL;
  struct coreInfo_t **core;
  SPICEdev **devs;
  Evt_Udn_Info_t  **udns;
  void *(*fetch)(void) = NULL;
  char *fullname, *pname;
  #ifdef DYNAMIC
  struct coreInfo_t *cinfo;

  xspicelib = DLOPEN("./libxspice"SUFFIX, RTLD_NOW);
  if(!xspicelib){
    //msg = dlerror();
    printf("%s\n", msg);
    return 1;
  }
  xspice_loaded = 1;
  #endif

  /* Initilise the structs and add digital node type */
  g_evt_udn_info = (Evt_Udn_Info_t  **)MALLOC(sizeof(Evt_Udn_Info_t  *));
  g_evt_num_udn_types = 1;

  #ifndef DYNAMIC
  g_evt_udn_info[0] =  &idn_digital_info;
  #else
  Evt_Udn_Info_t *pidn_digital_info;
  pidn_digital_info = (Evt_Udn_Info_t *) DLSYM(xspicelib, "idn_digital_info");

  g_evt_udn_info[0] =  pidn_digital_info;
  #endif

  DEVicesfl = (int *)tmalloc(DEVmaxnum*sizeof(int));
  /* tmalloc should automatically zero the array! */

  pname = strrchr(name, '.');
  if (!strcmp(pname, ".cm"))
    *pname = '\0';
  fullname = malloc(strlen(name) + 1 + 4);
  strcpy(fullname, name);
  strcat(fullname, SUFFIX);
  lib = DLOPEN(fullname, RTLD_NOW);
  if(!lib){
    //msg = dlerror();
    printf("%s\n", msg);
    return 1;
  }

  //fetch = dlsym(lib,"_CMdevNum");
  fetch = (void* (*)()) DLSYM(lib, PREFIX"CMdevNum");
  if(fetch){
    num = (int *)(*fetch)();
    printf("Got %u devices.\n",*num);
    fetch = NULL;
  }else{
    //msg = dlerror();
    printf("%s\n", msg);
    return 1;
  }

  //fetch = dlsym(lib,"_CMdevs");
  fetch = (void* (*)()) DLSYM(lib, PREFIX"CMdevs");
  if(fetch){
    devs = (SPICEdev **)(*fetch)();
    fetch = NULL;
  }else{
    //msg = dlerror();
    printf("%s\n", msg);
    return 1;
  }

  //fetch = dlsym(lib,"_CMgetCoreItfPtr");
  fetch = (void* (*)()) DLSYM(lib, PREFIX"CMgetCoreItfPtr");
  if(fetch){
    core = (struct coreInfo_t **)(*fetch)();
    #ifndef DYNAMIC
    *core = &coreInfo;
    #else
    cinfo = (struct coreInfo_t *) DLSYM(xspicelib, "coreInfo");
    *core = cinfo;
    #endif
    fetch = NULL;
  }else{
    //msg = dlerror();
    printf("%s\n", msg);
    return 1;
  }
  add_device(*num,devs,1);

  //fetch = dlsym(lib,"_CMudnNum");
  fetch = (void* (*)()) DLSYM(lib, PREFIX"CMudnNum");
  if(fetch){
    num = (int *)(*fetch)();
    printf("Got %u udns.\n",*num);
    fetch = NULL;
  }else{
    //msg = dlerror();
    printf("%s\n", msg);
    return 1;
  }

  //fetch = dlsym(lib,"_CMudns");
  fetch = (void* (*)()) DLSYM(lib, PREFIX"CMudns");
  if(fetch){
    udns = (Evt_Udn_Info_t  **)(*fetch)();
    fetch = NULL;
  }else{
    //msg = dlerror();
    printf("%s\n", msg);
    return 1;
  }

  add_udn(*num,udns);

  return 0;
}

EXPORT void com_codemodel(wordlist *wl)
{
  wordlist *ww;
  for(ww = wl; ww; ww = ww->wl_next)
    if(load_opus(wl->wl_word))
      fprintf(cp_err, "Error: Library %s couldn't be loaded!\n", ww->wl_word);
  return;
}

#endif  // XSPICE

