/**********
Copyright 2000 Regents of the University of California. All rights reserved.
Author: 2000 Weidong Liu
**********/

#include "spice.h"
#include <stdio.h>
#include "iferrmsg.h"
#include "util.h"
#include "inpdefs.h"
#include "strext.h"
#include "suffix.h"

/*ARGSUSED*/
EXPORT char *
INPdomodel(GENERIC *ckt, card *image, INPtables *tab)
{
    char *modname;
    int type = 0;
    int lev;
    char *typename;
    char *err = (char *)NULL;
    char *line;

    line = image->line;
    INPgetTok(&line,&modname,1);  /* throw away '.model' */
    INPgetTok(&line,&modname,1);
    INPinsert(&modname,tab);
    INPgetTok(&line,&typename,1);
    if( (strcmp(typename,"npn") == 0)||(strcmp(typename,"pnp")==0) ) {
	err = INPfindLev(line,&lev);
	switch(lev) {
		case 0:
		case 1:
			type = LoadModel("BJT");
			if (type < 0) {
	  		  err = INPmkTemp(
	  		          "Device type BJT not available in this binary\n");
			}
			break;
		case 2:
			type = LoadModel("BJT2");
                        if(type < 0) {
                          err = INPmkTemp(
                            "Device type BJT2 not available in this binary\n");
                        }
                        break;
		case 4:
		case 9:
			 type = LoadModel("VBIC");
                         if(type < 0) {
                           err = INPmkTemp(
                            "Device type VBIC not available in this binary\n");
                        }
                break;
		case 6:
			 type = LoadModel("MEXTRAM503");
                         if(type < 0) {
                           err = INPmkTemp(
                            "Device type MEXTRAM503 not available in this binary\n");
                        }
                break;
		case 7:
			 type = LoadModel("MEXTRAM504");
                         if(type < 0) {
                           err = INPmkTemp(
                            "Device type MEXTRAM504 not available in this binary\n");
                        }
                break;
		case 10:
			 type = LoadModel("MODELLA");
                         if(type < 0) {
                           err = INPmkTemp(
                            "Device type MODELLA not available in this binary\n");
                        }
                break;
		default: /* placeholder; use level 3 for the next model */
		err = INPmkTemp(
		  "Only BJT levels 1-2, 4, 6, 7, 9 are supported in this binary\n");
                break;
        }
        INPmakeMod(modname,type,image);
    } else if(strcmp(typename,"d") == 0) {
        type = LoadModel("Diode");
        if(type < 0) {
            err = INPmkTemp("Device type Diode not available in this binary\n");
        }
        INPmakeMod(modname,type,image);
    } else if( (strcmp(typename,"njf") == 0) || (strcmp(typename,"pjf") == 0)){
	err = INPfindLev(line, &lev);
	switch (lev) {
	case 0:
	case 1:
	    type = LoadModel("JFET");
	    if (type < 0) {
		err =
		    INPmkTemp
		    ("Device type JFET not available in this binary\n");
	    }
	    break;
	case 2:
	    type = LoadModel("JFET2");
	    if (type < 0) {
		err =
		    INPmkTemp
		    ("Device type JFET2 not available in this binary\n");
	    }
	    break;
	default:		/* placeholder; use level 3 for the next model */
	    err =
		INPmkTemp
		("Only JFET device levels 1-2 are supported in this binary\n");
	    break;
        }
        INPmakeMod(modname,type,image);
    } else if((strcmp(typename,"nmf") == 0) || (strcmp(typename,"pmf")==0) || 
              (strcmp(typename,"nhfet") == 0) || (strcmp(typename,"npfet")==0)) {
        err = INPfindLev(line,&lev);
        switch(lev) {
            case 0:
            case 1:
                type = LoadModel("MES");
                if(type < 0) {
                    err = INPmkTemp("Device type MES not available in this binary\n");
                }
                break;
            case 2:
                type = LoadModel("MESA");
                if(type < 0) {
                    err = INPmkTemp("Device type MESA not available in this binary\n");
                }
                break;
            case 3:
                type = LoadModel("MESA");
                if(type < 0) {
                    err = INPmkTemp("Device type MESA not available in this binary\n");
                }
                break;
            case 4:
                type = LoadModel("MESA");
                if(type < 0) {
                    err = INPmkTemp("Device type MESA not available in this binary\n");
                }
                break;
            case 5:
                type = LoadModel("HFET1");
                if(type < 0) {
                    err = INPmkTemp("Device type HFET1 not available in this binary\n");
                }
                break;
            case 6:
                type = LoadModel("HFET2");
                if(type < 0) {
                    err = INPmkTemp("Device type HFET2 not available in this binary\n");
                }
                break;
            default:
		err = INPmkTemp("Only MESFET device level 1 and HFET level 5-6 are supported in this binary\n");
                break;
        }
        INPmakeMod(modname,type,image);
    } else if(strcmp(typename,"urc") == 0) {
        type = LoadModel("URC");
        if(type < 0) {
            err = INPmkTemp("Device type URC not available in this binary\n");
        }
        INPmakeMod(modname,type,image);
    } else if( (strcmp(typename,"nmos")==0) || (strcmp(typename,"pmos")==0) ) {
        err = INPfindLev(line,&lev);
        switch(lev) {
            case 0:
            case 1:
                type = LoadModel("Mos1");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type MOS1 not available in this binary\n");
                }
                break;
            case 2:
                type = LoadModel("Mos2");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type MOS2 not available in this binary\n");
                }
                break;
            case 3:
                type = LoadModel("Mos3");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type MOS3 not available in this binary\n");
                }
                break;
            case 4:
                type = LoadModel("BSIM1");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type BSIM1 not available in this binary\n");
                }
                break;
            case 5:
                type = LoadModel("BSIM2");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type BSIM2 not available in this binary\n");
                }
                break;
            case 6:
                type = LoadModel("Mos6");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type MOS6 not available in this binary\n");
                }
                break;
            case 7:
                type = LoadModel("MOS7");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type MOS7 not available in this binary\n");
                }
                break;
            case 8:
                type = LoadModel("BSIM3");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type BSIM3 not available in this binary\n");
                }
                break;
            case 9:
                type = LoadModel("Mos9");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type Mos9 not available in this binary\n");
                }
                break;
            case 10:
                type = LoadModel("B3SOI");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type B3SOI not available in this binary\n");
                }
                break;
            case 14:
                type = LoadModel("BSIM4");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type BSIM4 not available in this binary\n");
                }
                break;
            case 15:
                type = LoadModel("BSIM5");
                if(type < 0) {
                    err = INPmkTemp(
                            "Placeholder: Device type BSIM5 not available in this binary\n");
                }
                break;
            case 16:
                type = LoadModel("BSIM6");
                if(type < 0) {
                    err = INPmkTemp(
                            "Placeholder: Device type BSIM6 not available in this binary\n");
                }
                break;
	    case 29:
	        type = LoadModel("B3SOIPD");
  	        if (type < 0) {
		    err = INPmkTemp(
		            "Placeholder: Device type B3SOIPD not available in this binary\n");
	        }
	        break;
            case 30:
	        type = LoadModel("B3SOIFD");
	        if (type < 0) {
		    err = INPmkTemp(
		            "Placeholder: Device type B3SOIFD not available in this binary\n");
	        }
	        break;
            case 31:
	        type = LoadModel("B3SOIDD");
	        if (type < 0) {
		    err = INPmkTemp(
		            "Placeholder: Device type B3SOIDD not available in this binary\n");
	        }
	        break;
            case 44:
		type = LoadModel("EKV");
		if (type < 0) {
		err = INPmkTemp
		    ("Placeholder for EKV model: look at http://legwww.epfl.ch for info on EKV\n");
	        }
	    break;    
            case 60:
                type = LoadModel("HiSIM1");
                if(type < 0) {
                    err = INPmkTemp(
                            "Device type HiSIM1 not available in this binary\n");
                }
                break;
            default: /* Use level 17 or higher levels for the other models */
		err = INPmkTemp(
		  "Only MOS device levels 1-14 and 60 are supported in this binary\n");
                break;
        }
        INPmakeMod(modname,type,image);
    } else if(strcmp(typename,"r") == 0) {
        type = LoadModel("Resistor");
        if(type < 0) {
            err = INPmkTemp(
                    "Device type Resistor not available in this binary\n");
        }
        INPmakeMod(modname,type,image);
    } else if(strcmp(typename,"c") == 0) {
        type = LoadModel("Capacitor");
        if(type < 0) {
            err = INPmkTemp(
                    "Device type Capacitor not available in this binary\n");
        }
        INPmakeMod(modname,type,image);
    } else if(strcmp(typename,"sw") == 0) {
        type = LoadModel("Switch");
        if(type < 0) {
            err = INPmkTemp(
                    "Device type Switch not available in this binary\n");
        }
        INPmakeMod(modname,type,image);
    } else if(strcmp(typename,"csw") == 0) {
        type = LoadModel("CSwitch");
        if(type < 0) {
            err = INPmkTemp(
                    "Device type CSwitch not available in this binary\n");
        }
        INPmakeMod(modname,type,image);
    } else if(strcmp(typename,"ltra") == 0) {
        type = LoadModel("LTRA");
        if(type < 0) {
            err = INPmkTemp(
                    "Device type LTRA not available in this binary\n");
        }
        INPmakeMod(modname,type,image);
    } else {

#ifndef XSPICE
        type = -1;
        err = (char *)MALLOC(35 + strlen(typename));
        (void)sprintf(err,"unknown model type %s - ignored\n",typename);
#else	
      /* gtri - modify - wbk - 10/23/90 - modify to look for code models */

      /* look for this model type and put it in the table of models */
      type = INPtypelook(typename);
      if(type < 0) {
	err = (char *) MALLOC(35 + strlen(typename));
	sprintf(err,"Unknown model type %s - ignored\n",typename);
      }
      else {
	INPmakeMod(modname,type,image);
      }
      
      /* gtri - end - wbk - 10/23/90 */
#endif

    }
    return(err);
}
