/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1985 Thomas L. Quarles
**********/

/*  look up the 'type' in the device description struct and return the
 *  appropriate index for the device found, or -1 for not found
 */

#include "spice.h"
#include "strext.h"
#include "inpdefs.h"
#include "cpdefs.h"
#include "fteext.h"
#include "ifsim.h"
#include "suffix.h"

#if (defined DYNAMIC) || (defined DXE)
#ifndef linux
#define MODPATH "models\\"
#else
#define MODPATH "/models/"
#endif

#include "modlib.h"
#endif

#if (defined DYNAMIC) && (defined WIN32)
#include <windows.h>
#include "devdefs.h"
#include "spicedev.h"

EXPORT int
LoadModel(char *type)
{

    int i;
    HANDLE dllHandle;
    SPICEdev ***p;
    char modelpath[256] = MODPATH;
    char errmsg[256];

    for(i=0;i<ft_sim->numDevices;i++)
    //for(i=0;i<MyModmaxnum;i++)
        {
        if(strcmp(type, MyModLib[i].name)==0)
            {
            // Load model library
            if ((dllHandle = LoadLibrary(strcat(modelpath, MyModLib[i].library))) == NULL)
                {
                sprintf(errmsg, "Can't load %s library", MyModLib[i].library);
                MessageBox(NULL, errmsg, "Spice+", MB_OK+MB_ICONWARNING);
                exit(-1);
                }
            p = &DEVices;
            *(*p+i) = (SPICEdev *) GetProcAddress(dllHandle, MyModLib[i].infoname);

            // found the device - return it
            return(i);
            }
        }

    return(-1);
}
#endif
#ifdef DXE
#include "devdefs.h"
#include "spicedev.h"
#include "dlfcn.h"

EXPORT int
LoadModel(char *type)
{

    int i;
    void *handle;
    SPICEdev ***p;
    char modelpath[256] = MODPATH;

    for(i=0;i<ft_sim->numDevices;i++)
        {
        if(strcmp(type, MyModLib[i].name)==0)
            {
            // Load model library
            if ((handle = dlopen(strcat(modelpath, MyModLib[i].library), 0)) == NULL)
                {
                printf("Can't load library %s\n", modelpath);
                exit(-1);
                }
            p = &DEVices;
            *(*p+i) = (SPICEdev *) dlsym(handle, MyModLib[i].infoname);

            // found the device - return it
            return(i);
            }
        }

    return(-1);
}
#endif
#if (defined DYNAMIC) && (defined linux)
#include "devdefs.h"
#include "spicedev.h"
#include "dlfcn.h"

EXPORT int
LoadModel(char *type)
{

    int i;
    void *handle;
    SPICEdev ***p;
    //char modelpath[256] = MODPATH;
    char modelpath[256];
    char *error;

    strcpy(modelpath, Spice_Exec_Dir);
    strcat(modelpath, MODPATH);
    for(i=0;i<ft_sim->numDevices;i++)
        {
        if(strcmp(type, MyModLib[i].name)==0)
            {
            // Load model library
            if ((handle = dlopen(strcat(modelpath, MyModLib[i].library), RTLD_NOW)) == NULL)
                {
                error = dlerror();
                printf("Can't load %s library\n", MyModLib[i].library);
                printf("%s\n", error);
                exit(-1);
                }
            p = &DEVices;
            *(*p+i) = (SPICEdev *) dlsym(handle, MyModLib[i].infoname);

            // found the device - return it
            return(i);
            }
        }

    return(-1);
}
#endif
#if !(defined DYNAMIC) && !(defined DXE)
EXPORT int
LoadModel(char *type)
{

    int i;
    for(i=0;i<ft_sim->numDevices;i++) {
        if(strcmp(type,(*(ft_sim->devices)[i]).name)==0) {
            /*found the device - return it */
            return(i);
        }
    }
    return(-1);
}
#endif

EXPORT int
INPtypelook(char *type)
{

    int i;
    for(i=0;i<ft_sim->numDevices;i++)
        {
        if((ft_sim->devices)[i] != NULL)
        if(strcmp(type,(*(ft_sim->devices)[i]).name)==0) {
            /*found the device - return it */
            return(i);
        }
    }
    return(-1);
}

