/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1987 Wayne A. Christopher, U. C. Berkeley CAD Group 
**********/

/*
 * All the functions used in the parse tree.  These functions return HUGE
 * if their argument is out of range.
 */

#include "spice.h"
#include <stdio.h>
#include "ifsim.h"
#include "inpptree.h"
#include "suffix.h"

/* XXX These should be in math.h */

#ifdef HAS_ATRIGH
#  ifdef HAS_NO_ATRIGH_DECL
extern double asinh(), acosh(), atanh();
#  endif
#endif /* HAS_ATRIGH */

double PTfudge_factor;

#define	MODULUS(NUM,LIMIT)	((NUM) - ((int) ((NUM) / (LIMIT))) * (LIMIT))

EXPORT double
PTabs(double arg)
{
    return arg >= 0.0 ? arg : -arg;
}

EXPORT double
PTsgn(double arg)
{
    return arg > 0.0 ? 1.0 : arg < 0.0 ? -1.0 : 0.0;
}

EXPORT double
PTplus(double arg1, double arg2)
{
    return (arg1 + arg2);
}

EXPORT double
PTminus(double arg1, double arg2)
{
    return (arg1 - arg2);
}

EXPORT double
PTtimes(double arg1, double arg2)
{
    return (arg1 * arg2);
}

EXPORT double 
PTdivide(double arg1, double arg2)
{
    if (arg2 >= 0.0)
	arg2 += PTfudge_factor;
    else
	arg2 -= PTfudge_factor;

    if (arg2 == 0.0)
        return (HUGE);

    return (arg1 / arg2);
}

EXPORT double
PTpower(double arg1, double arg2)
{
    if (arg1 < 0.0) {
	if (fabs(arg2 - ((int) arg2)) / (arg2 + 0.001) < 0.000001) {
	    arg2 = (int) arg2;
	} else {
	    arg1 = -arg1;
	}
    }
    return (pow(arg1, arg2));
}

EXPORT double
PTacos(double arg)
{
    return (acos(arg));
}

EXPORT double
PTacosh(double arg)
{
#ifdef HAS_ATRIGH
    return (acosh(arg));
#else
    if (arg < 1.0)
	arg = 1.0;
    return (log(arg + sqrt(arg*arg-1.0)));
#endif /* HAS_ATRIGH */
}

EXPORT double
PTasin(double arg)
{
    return (asin(arg));
}

EXPORT double
PTasinh(double arg)
{
#ifdef HAS_ATRIGH
    return (asinh(arg));
#else
    return log(arg + sqrt(arg * arg + 1.0));
#endif /* HAS_ATRIGH */
}

EXPORT double
PTatan(double arg)
{
    return (atan(arg));
}

EXPORT double
PTatanh(double arg)
{
#ifdef HAS_ATRIGH
    return (atanh(arg));
#else
    if (arg < -1.0)
	arg = -1.0 + PTfudge_factor + 1e-10;
    else if (arg > 1.0)
	arg = 1.0 - PTfudge_factor - 1e-10;
    return (log((1.0 + arg) / (1.0 - arg)) / 2.0);
#endif /* HAS_ATRIGH */
}

EXPORT double
PTustep(double arg)
{
    if (arg < 0.0)
	return 0.0;
    else if (arg > 0.0)
	return 1.0;
    else
	return 0.5; /* Ick! */
}

EXPORT double
PTuramp(double arg)
{
    if (arg < 0.0)
	return 0.0;
    else
	return arg;
}

EXPORT double
PTcos(double arg)
{
    return (cos(MODULUS(arg, 2 * M_PI)));
}

EXPORT double
PTcosh(double arg)
{
    return (cosh(arg));
}

EXPORT double
PTexp(double arg)
{
    return (exp(arg));
}

EXPORT double
PTln(double arg)
{
    if (arg < 0.0)
	arg = -arg;
    return (log(arg));
}

EXPORT double
PTlog(double arg)
{
    if (arg < 0.0)
	arg = -arg;
    return (log10(arg));
}

EXPORT double
PTsin(double arg)
{
    return (sin(MODULUS(arg, 2 * M_PI)));
}

EXPORT double
PTsinh(double arg)
{
    return (sinh(arg));
}

EXPORT double
PTsqrt(double arg)
{
    if (arg < 0.0)
	arg = -arg;
    return (sqrt(arg));
}

EXPORT double
PTtan(double arg)
{
    return (tan(MODULUS(arg, M_PI)));
}

EXPORT double
PTtanh(double arg)
{
    return (tanh(arg));
}

EXPORT double
PTuminus(double arg)
{
    return (- arg);
}

