/**********
Copyright (c) 2003-2004 Paolo Fallucca.
**********/

#include "spice.h"

#include "mfb.h"
#include "ftedev.h"
#include "ftedefs.h"
#include "ftegraph.h"
#include "mfbpc.h"

#ifdef GDI
#define redraw() InvalidateRect(hwnd, NULL, false)

#define blit(srcbmp,destbmp,src_x,src_y,dest_x,dest_y,width,height) \
        BitBlt(destbmp,dest_x,dest_y,width,height,srcbmp,src_x,src_y,SRCCOPY)
extern DWORD dwRop;
extern HWND hwndDlg;
#endif

#ifdef GTK
#define blit(srcbmp,destbmp,src_x,src_y,dest_x,dest_y,width,height) \
        gdk_draw_pixmap(destbmp,drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],srcbmp,src_x,src_y,dest_x,dest_y,width,height)
extern GtkWidget *curs_window;
extern char c1str[40],c2str[40],difstr[40];
extern GtkWidget *c1str_label, *c2str_label, *difstr_label;
#endif

extern int num_plot;
//extern void setcolor();

// Cursor Position
int c1_screenx,c1_screeny;
int c2_screenx,c2_screeny;
int c1_xpos,c1_ypos;
int c2_xpos,c2_ypos;
int c1_currtrace;
int c2_currtrace;

void rev_vect(double *xvect, double *yvect, int length)
{
int i;
double temp;

for (i=0; i<length/2; i++)
    {
    temp = xvect[i];
    xvect[i] = xvect[length-1-i];
    xvect[length-1-i] = temp;

    temp = yvect[i];
    yvect[i] = yvect[length-1-i];
    yvect[length-1-i] = temp;
    }
}

void get_startpos(double *x, double *y, int ctrace)
// Return the x and y coordinates of the first point calculated of the
// trace number ctrace.
{
int i;
double lx,rx,ly,ry;
struct dvec *v, *xs;
struct pnode *names,*n;

    // Evaluate the vector.
    v = NULL;
    names = ft_getpnames((*curr_dataplot).tracelist, true);
    i = 0;
    for (n=names; n; n = n->pn_next)
        {
        if (i==ctrace)
            v = ft_evaluate(n);
        i++;
        }
    
    // Now fill in the scale.
    if (!v->v_scale)
        {
        if (v->v_plot->pl_scale)
            v->v_scale = v->v_plot->pl_scale;
        else
            v->v_scale = v;
        }
    //xs = v->v_scale;
    xs = vec_copy(v->v_scale);

// Check if scale is reversed
lx = isreal(xs) ? xs->v_realdata[0] : realpart(&xs->v_compdata[0]);
rx = isreal(xs) ? xs->v_realdata[xs->v_length-1] : realpart(&xs->v_compdata[xs->v_length-1]);
if (lx > rx)
    rev_vect(xs->v_realdata, v->v_realdata, xs->v_length);

// Polar
if ((((*curr_dataplot).graphic)->grid.gridtype) == GRID_POLAR)
    {
    rx = isreal(v) ? v->v_realdata[i] : realpart(&v->v_compdata[i]);
    i = 0;
    while (rx < (((*curr_dataplot).graphic)->datawindow.xmin))
        {
        i++;
        rx = isreal(v) ? v->v_realdata[i] : realpart(&v->v_compdata[i]);
        }
        
    *x = rx;
    *y = isreal(v) ? v->v_realdata[i] : imagpart(&v->v_compdata[i]);
    }
else
    {
    *x = ((*curr_dataplot).graphic)->datawindow.xmin;

    // x is on the left of calculated points
    lx = isreal(xs) ? xs->v_realdata[0] : realpart(&xs->v_compdata[0]);
    if (*x < lx)
        *x = lx;
    
    rx = isreal(xs) ? xs->v_realdata[0] : realpart(&xs->v_compdata[0]);
    for (i = 1; (rx < (((*curr_dataplot).graphic)->datawindow.xmin)) && (i < v->v_length); i++)
        rx = isreal(xs) ? xs->v_realdata[i] : realpart(&xs->v_compdata[i]);
        
    if (i==1)
        *y = isreal(v) ? v->v_realdata[0] : realpart(&v->v_compdata[0]);
    else
        {
        lx = isreal(xs) ? xs->v_realdata[i-2] : realpart(&xs->v_compdata[i-2]);
        ly = isreal(v) ? v->v_realdata[i-2] : realpart(&v->v_compdata[i-2]);
        ry = isreal(v) ? v->v_realdata[i-1] : realpart(&v->v_compdata[i-1]);
        *y = (*x-lx)/(rx-lx)*(ry-ly)+ly;
        }
    }
}

void Set_plot_rect(SPICE_RECT *rectgr, GRAPH *graph)
// Return the coordinates of the rectangular region in which the graph
// is plotted
{
rectgr->x = graph->viewportxoff;
rectgr->y = config.numypixels + menu_width - graph->viewportyoff - graph->viewport.height;
rectgr->w = graph->viewport.width;
rectgr->h = graph->viewport.height;
}

void get_ycoor(double *x, double *y, int ctrace)
{
int i;
double x1,y1,y2;
double lx,rx,ly,ry;
double olddiff,newdiff;
struct dvec *v, *xs;
struct pnode *names,*n;

    // Evaluate the vector.
    v = NULL;
    names = ft_getpnames((*curr_dataplot).tracelist, true);
    i = 0;
    for (n=names; n; n = n->pn_next)
        {
        if (i==ctrace)
            v = ft_evaluate(n);
        i++;
        }
    
    // Now fill in the scale.
    if (!v->v_scale)
        {
        if (v->v_plot->pl_scale)
            v->v_scale = v->v_plot->pl_scale;
        else
            v->v_scale = v;
        }
    //xs = v->v_scale;
    xs = vec_copy(v->v_scale);

// Check if scale is reversed
lx = isreal(xs) ? xs->v_realdata[0] : realpart(&xs->v_compdata[0]);
rx = isreal(xs) ? xs->v_realdata[xs->v_length-1] : realpart(&xs->v_compdata[xs->v_length-1]);
if (lx > rx)
    rev_vect(xs->v_realdata, v->v_realdata, xs->v_length);

x1 = *x;

// Polar
if ((((*curr_dataplot).graphic)->grid.gridtype) == GRID_POLAR)
    {
    lx = isreal(v) ? v->v_realdata[0] : realpart(&v->v_compdata[0]);
    rx = lx;
    olddiff = fabs(rx-(*x));
    for (i = 1; i < v->v_length; i++)
        {
        rx = isreal(v) ? v->v_realdata[i] : realpart(&v->v_compdata[i]);
        newdiff = fabs(rx-(*x));
        if (newdiff > olddiff)
            break;
        olddiff = newdiff;
        }

    if (i==1)
        y1 = isreal(v) ? v->v_realdata[0] : imagpart(&v->v_compdata[0]);
    else
        {
        lx = isreal(v) ? v->v_realdata[i-2] : realpart(&v->v_compdata[i-2]);
        ly = isreal(v) ? v->v_realdata[i-2] : imagpart(&v->v_compdata[i-2]);
        ry = isreal(v) ? v->v_realdata[i-1] : imagpart(&v->v_compdata[i-1]);
        y1 = (*x-lx)/(rx-lx)*(ry-ly)+ly;
        }
    }
else
    {
    // External cycle    
    lx = isreal(xs) ? xs->v_realdata[0] : realpart(&xs->v_compdata[0]);
    
    // x is on the left of calculated points
    if (*x < lx)
        x1 = lx;
        
    rx = lx;
    for (i = 1; (rx < *x) && (i < v->v_length) && (rx-lx>=0); i++)
        {
        lx = isreal(xs) ? xs->v_realdata[i-1] : realpart(&xs->v_compdata[i-1]);
        rx = isreal(xs) ? xs->v_realdata[i] : realpart(&xs->v_compdata[i]);
        }
        
    if (i==1)
        y1 = isreal(v) ? v->v_realdata[0] : realpart(&v->v_compdata[0]);
    // x is on the right of calculated points
    else if (*x > rx)
        {
        x1 = isreal(xs) ? xs->v_realdata[i-2] : realpart(&xs->v_compdata[i-2]);
        y1 = isreal(v) ? v->v_realdata[i-2] : realpart(&v->v_compdata[i-2]);
        }
    else
        {
        lx = isreal(xs) ? xs->v_realdata[i-2] : realpart(&xs->v_compdata[i-2]);
        ly = isreal(v) ? v->v_realdata[i-2] : realpart(&v->v_compdata[i-2]);
        ry = isreal(v) ? v->v_realdata[i-1] : realpart(&v->v_compdata[i-1]);
        y1 = (*x-lx)/(rx-lx)*(ry-ly)+ly;
        }

    // Check for others y
	i++;
    lx = isreal(xs) ? xs->v_realdata[i] : realpart(&xs->v_compdata[i]);
    while ((*x < lx) && (i < v->v_length))
        {
        lx = isreal(xs) ? xs->v_realdata[i] : realpart(&xs->v_compdata[i]);
		i++;
        }

	i++;
    while (i < v->v_length)
        {
        rx = lx;
        for (; (rx < *x) && (i < v->v_length) && (rx-lx>=0); i++)
            {
            lx = isreal(xs) ? xs->v_realdata[i-1] : realpart(&xs->v_compdata[i-1]);
            rx = isreal(xs) ? xs->v_realdata[i] : realpart(&xs->v_compdata[i]);
            }

        if (rx < *x)
            y2 = isreal(v) ? v->v_realdata[i-2] : realpart(&v->v_compdata[i-2]);
        else
            {
            lx = isreal(xs) ? xs->v_realdata[i-2] : realpart(&xs->v_compdata[i-2]);
            ly = isreal(v) ? v->v_realdata[i-2] : realpart(&v->v_compdata[i-2]);
            ry = isreal(v) ? v->v_realdata[i-1] : realpart(&v->v_compdata[i-1]);
            y2 = (*x-lx)/(rx-lx)*(ry-ly)+ly;
            }
            
        if (fabs(*y-y2) < fabs(*y-y1))
            y1 = y2;

        lx = isreal(xs) ? xs->v_realdata[i] : realpart(&xs->v_compdata[i]);
        while ((*x < lx) && (i < v->v_length))
            {
            i++;
            lx = isreal(xs) ? xs->v_realdata[i] : realpart(&xs->v_compdata[i]);
            }
		i++;
        }
    }
*x = x1;
*y = y1;
}

int Find_cur_pos(int *screenx, int *screeny, int *xpos, int *ypos, int ctrace)
{
double x,y;
double oldx;

ScreentoData((*curr_dataplot).graphic,*screenx,*screeny,&x,&y);
oldx = x;
    
get_ycoor(&x,&y,ctrace);
DatatoScreen((*curr_dataplot).graphic,x,y,xpos,ypos);
if (oldx==x)
    *xpos = *screenx;
else
    *screenx = *xpos;

return 0;
}

int Check_plot_click(int x_mousepos, int y_mousepos, int cplot)
// Check if the current plot has been clicked (mouse coordinates inside
// the graph). In this case return 1, otherwise 0.
{
if ((x_mousepos > dataplot[cplot].rgraph.x - 1) && (x_mousepos < dataplot[cplot].rgraph.x + dataplot[cplot].rgraph.w + 1) &&
    (y_mousepos > dataplot[cplot].rgraph.y + config.numypixels * cplot - 1) && (y_mousepos < dataplot[cplot].rgraph.y + dataplot[cplot].rgraph.h + config.numypixels * cplot + 1))
    {
    return 1;
    }
return 0;
}

void Transform_coord(int x_mousepos, int y_mousepos,
                     int *screenx, int *screeny, int cplot)
// Set screenx and screeny, changing the origin af axis in the lower left
// coorner.
{
*screenx = x_mousepos;
*screeny = num_plot * config.numypixels - (y_mousepos - cplot * config.numypixels) + menu_width;
}

char findunit(double *x)
{
static char scaleletters[] = "afpnum kMGT";
int i;
char s;

i = 6;
if (*x != 0)
    {
    while (fabs(*x) >= 1000)
        {
        *x = *x / 1000;
        i++;
        }
    while (fabs(*x) < 0.01)
        {
        *x = *x * 1000;
        i--;
        }
    }
s = scaleletters[i];
return s;
}

SPICE_RECT find_legend_pos(int plotno)
{
SPICE_RECT legrect;

legrect.x = ((plotno % 2) ? ((*curr_dataplot).graphic)->viewportxoff :
        ((((*curr_dataplot).graphic)->viewport.width) / 2));
legrect.y = ((*curr_dataplot).graphic)->absolute.height - ((*curr_dataplot).graphic)->fontheight
        - ((plotno + 2) / 2) * ((*curr_dataplot).graphic)->fontheight
        + ((*curr_dataplot).graphic)->fontheight + 1;
legrect.w = ((*curr_dataplot).graphic)->viewport.width / 20;
legrect.h = ((*curr_dataplot).graphic)->fontheight;

return legrect;
}

int Check_legend_click(int x_mousepos, int y_mousepos, int *ctrace, int cplot)
{
int i;
SPICE_RECT legrect;

for (i = 0; i < (*curr_dataplot).ntrace; i++)
    {
    legrect = find_legend_pos(i);
    if ((x_mousepos > legrect.x - 1) && (x_mousepos < legrect.x + legrect.w + 1) &&
        (y_mousepos > config.numypixels - legrect.y + config.numypixels * cplot - 1) && (y_mousepos < config.numypixels - legrect.y + legrect.h + config.numypixels * cplot + menu_width + 1))
        {
        *ctrace = i;
        return 1;
        }
    }
return 0;
}

void get_cursor_init_pos()
{
double x1,y1;
double x2,y2;

c1_currtrace = 0;
c2_currtrace = 0;

// Set cursor1 position to xmin
get_startpos(&x1,&y1,c1_currtrace);
DatatoScreen((*curr_dataplot).graphic,x1,y1,&c1_xpos,&c1_ypos);
c1_screenx = c1_xpos;
c1_screeny = c1_ypos;

// The same for cursor2
get_startpos(&x2,&y2,c2_currtrace);
DatatoScreen((*curr_dataplot).graphic,x2,y2,&c2_xpos,&c2_ypos);
c2_screenx = c2_xpos;
c2_screeny = c2_ypos;

// Put in bmp che current plot taken from plotsbmp
blit(plotsbmp,bmp,0,config.numypixels * curr_plot,0,0,
     config.numxpixels,config.numypixels);
}

void dotted_rect(int x1,int y1,int x2,int y2)
{
MFBHLine(x1, y1, x2);
MFBHLine(x1, y2, x2);
MFBVLine(x1, y1, y2);
MFBVLine(x2, y1, y2);
}

void draw_cursor()
{
SPICE_RECT legrect;

blit(bmp,bmptmp,0,0,0,0,config.numxpixels,config.numypixels);

// Cursor color
SetColor(20);

#ifdef GDI
dwRop = PATINVERT;
#endif

#ifdef GTK
gdk_gc_set_function(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], GDK_OR);
#endif

currbmp = bmptmp;

// Cursor 1
// Draw cursor position
SetLinestyle(1);
if ((c1_ypos<config.numypixels + menu_width - curr_dataplot->rgraph.y + 1) &&
    (c1_ypos>config.numypixels + menu_width - curr_dataplot->rgraph.y - curr_dataplot->rgraph.h - 1))
    MFBHLine(curr_dataplot->rgraph.x, c1_ypos, curr_dataplot->rgraph.x + curr_dataplot->rgraph.w);
MFBVLine(c1_xpos, config.numypixels + menu_width - curr_dataplot->rgraph.y,
         config.numypixels + menu_width - curr_dataplot->rgraph.y - curr_dataplot->rgraph.h);
// Draw legend selection rect
legrect = find_legend_pos(c1_currtrace);
dotted_rect(legrect.x, legrect.y,
     legrect.x + legrect.w,
     legrect.y - legrect.h);

// Cursor 2
// Draw cursor position
SetLinestyle(8);
if ((c2_ypos<config.numypixels + menu_width - curr_dataplot->rgraph.y + 1) &&
    (c2_ypos>config.numypixels + menu_width - curr_dataplot->rgraph.y - curr_dataplot->rgraph.h - 1))
    MFBHLine(curr_dataplot->rgraph.x, c2_ypos, curr_dataplot->rgraph.x + curr_dataplot->rgraph.w);
MFBVLine(c2_xpos, config.numypixels + menu_width - curr_dataplot->rgraph.y,
         config.numypixels + menu_width - curr_dataplot->rgraph.y - curr_dataplot->rgraph.h);
// Draw legend selection rect
legrect = find_legend_pos(c2_currtrace);
dotted_rect(legrect.x, legrect.y,
     legrect.x + legrect.w,
     legrect.y - legrect.h);

#ifdef GTK
gdk_gc_set_function(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], GDK_COPY);
#endif

blit(bmptmp,plotsbmp,0,0,0,config.numypixels * curr_plot,config.numxpixels,config.numypixels);

// Reset line style
SetLinestyle(0);

// Reset the current bitmap
currbmp = bmp;

#ifdef GDI
BitBlt(plotsbmp,0,config.numypixels * curr_plot,config.numxpixels,config.numypixels,bmptmp,0,0,SRCCOPY);
redraw();
InvalidateRect(hwndDlg, NULL, false);
#endif

#ifdef GTK
plot_curs_pos(c1str, c2str, difstr);
gtk_label_set_text(GTK_LABEL(c1str_label), c1str);
gtk_label_set_text(GTK_LABEL(c2str_label), c2str);
gtk_label_set_text(GTK_LABEL(difstr_label), difstr);
gdk_window_invalidate_rect(curs_window->window, NULL, TRUE);
#endif
}

void plot_curs_pos(char *c1str, char *c2str, char *difstr)
{
double x1,y1;
double x2,y2;
double difx,dify;
char x1u,y1u,x2u,y2u,dxu,dyu;

ScreentoData(curr_dataplot->graphic,c1_xpos,c1_ypos,&x1,&y1);
ScreentoData(curr_dataplot->graphic,c2_xpos,c2_ypos,&x2,&y2);
get_ycoor(&x1,&y1,c1_currtrace);
get_ycoor(&x2,&y2,c2_currtrace);

difx = x1-x2;
dify = y1-y2;
x1u = findunit(&x1);
y1u = findunit(&y1);
x2u = findunit(&x2);
y2u = findunit(&y2);
dxu = findunit(&difx);
dyu = findunit(&dify);
sprintf( c1str,"C1 = %11.5G%c , %11.5G%c", x1, x1u, y1, y1u);
sprintf( c2str,"C2 = %11.5G%c , %11.5G%c", x2, x2u, y2, y2u);
sprintf(difstr,"dif= %11.5G%c , %11.5G%c", difx, dxu, dify, dyu);
}

