/**********
Copyright (c) 2003-2004 Paolo Fallucca.
**********/

#include "spice.h"

#ifdef HAS_MFB
#include "allegro.h"

int fixed_button_proc(int msg, DIALOG *d, int c);

int button_dp2_proc(int msg, DIALOG *d, int c)
{
    int ret;
    int (*proc)(DIALOG *dl);

    // call the parent object
    ret = fixed_button_proc(msg, d, c);

    // trap the close return value
    if (ret==D_CLOSE)
    {
    // call the sub-function
    if (d->dp2)
    {
        proc = d->dp2;
        return (*proc)(d);
    } else
        return D_REDRAW;
    }
    return ret;
}

/*  dotted_rect:
 *  Draws a dotted rectangle, for showing an object has the input focus.
 */
static void dotted_rect(int x1, int y1, int x2, int y2, int fg, int bg)
{
   int x = ((x1+y1) & 1) ? 1 : 0;
   int c;

   for (c=x1; c<=x2; c++) {
      putpixel(screen, c, y1, (((c+y1) & 1) == x) ? fg : bg);
      putpixel(screen, c, y2, (((c+y2) & 1) == x) ? fg : bg);
   }

   for (c=y1+1; c<y2; c++) {
      putpixel(screen, x1, c, (((c+x1) & 1) == x) ? fg : bg);
      putpixel(screen, x2, c, (((c+x2) & 1) == x) ? fg : bg);
   }
}

/*  fixed_button_proc:
 *  A button object (the dp field points to the text string). This object
 *  can be selected by clicking on it with the mouse or by pressing its 
 *  keyboard shortcut. If the D_EXIT flag is set, selecting it will close 
 *  the dialog, otherwise it will toggle on and off.
 */
int fixed_button_proc(int msg, DIALOG *d, int c)
{
   int state1, state2;
   //int black;
   int swap;
   int g;
   int rtm;

   switch (msg) {

      case MSG_DRAW:
     if (d->flags & D_SELECTED) {
        g = 1;
        state1 = d->bg;
        state2 = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
     }
     else {
        g = 0;
        state1 = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
        state2 = d->bg;
     }

     rectfill(screen, d->x+1+g, d->y+1+g, d->x+d->w-3+g, d->y+d->h-3+g, state2);
     rect(screen, d->x+g, d->y+g, d->x+d->w-2+g, d->y+d->h-2+g, state1);
     rtm = text_mode(-1);
     gui_textout(screen, d->dp, d->x+d->w/2+g, d->y+d->h/2-text_height(font)/2+g, state1, TRUE);
     text_mode(rtm);

     if (d->flags & D_SELECTED) {
        vline(screen, d->x, d->y, d->y+d->h-2, d->bg);
        hline(screen, d->x, d->y, d->x+d->w-2, d->bg);
     }
     else {
        //black = makecol(0,0,0);
        vline(screen, d->x+d->w-1, d->y+1, d->y+d->h-2, d->fg);
        hline(screen, d->x+1, d->y+d->h-1, d->x+d->w-1, d->fg);
     }
     if ((d->flags & D_GOTFOCUS) &&
         (!(d->flags & D_SELECTED) || !(d->flags & D_EXIT)))
            dotted_rect(d->x+1+g, d->y+1+g, d->x+d->w-3+g, d->y+d->h-3+g, state1, state2);
     break;

      case MSG_WANTFOCUS:
     return D_WANTFOCUS;

      case MSG_KEY:
     /* close dialog? */
     if (d->flags & D_EXIT) {
        return D_CLOSE;
     }

     /* or just toggle */
     d->flags ^= D_SELECTED;
     scare_mouse();
     SEND_MESSAGE(d, MSG_DRAW, 0);
     unscare_mouse();
     break;

      case MSG_CLICK:
     /* what state was the button originally in? */
     state1 = d->flags & D_SELECTED;
     if (d->flags & D_EXIT)
        swap = FALSE;
     else
        swap = state1;

     /* track the mouse until it is released */
     while (gui_mouse_b()) {
        state2 = ((gui_mouse_x() >= d->x) && (gui_mouse_y() >= d->y) &&
                  (gui_mouse_x() < d->x + d->w) && (gui_mouse_y() < d->y + d->h));
        if (swap)
           state2 = !state2;

        /* redraw? */
        if (((state1) && (!state2)) || ((state2) && (!state1))) {
           d->flags ^= D_SELECTED;
           state1 = d->flags & D_SELECTED;
           scare_mouse();
           SEND_MESSAGE(d, MSG_DRAW, 0);
           unscare_mouse();
        }

        /* let other objects continue to animate */
        broadcast_dialog_message(MSG_IDLE, 0);
     }

     /* should we close the dialog? */
     if ((d->flags & D_SELECTED) && (d->flags & D_EXIT)) {
        d->flags ^= D_SELECTED;
        return D_CLOSE;
     }
     break;
   }
   
   return D_O_K;
}

void fixed_do_arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int d, void (*proc)(BITMAP *, int, int, int))
{
   unsigned long rr = r*r;
   unsigned long rr1, rr2, rr3;
   int px, py;
   int ex, ey;
   int px1, px2, px3;
   int py1, py2, py3;
   int d1, d2, d3;
   int ax, ay;
   int q, qe;
   long tg_cur, tg_end;
   int done = FALSE;

   rr1 = r;
   rr2 = itofix(x);
   rr3 = itofix(y);

   /* evaluate the start point and the end point */
   px = fixtoi(rr2 + rr1 * fcos(ang1));
   py = fixtoi(rr3 - rr1 * fsin(ang1));
   ex = fixtoi(rr2 + rr1 * fcos(ang2));
   ey = fixtoi(rr3 - rr1 * fsin(ang2));

   /* start quadrant */
   if (px >= x) {
      if (py <= y)
     q = 1;                           /* quadrant 1 */
      else
     q = 4;                           /* quadrant 4 */
   }
   else {
      if (py < y)
     q = 2;                           /* quadrant 2 */
      else
     q = 3;                           /* quadrant 3 */
   }

   /* end quadrant */
   if (ex >= x) {

// !!! Changed 'ey <= y' with 'ey < y', so, if starting point coincide 
// with ending point, 'qe = 4' and a circle will be drawn. 
      if (ey < y)
     qe = 1;                          /* quadrant 1 */
      else
     qe = 4;                          /* quadrant 4 */
   }
   else {
      if (ey < y)
     qe = 2;                          /* quadrant 2 */
      else
     qe = 3;                          /* quadrant 3 */
   }

   #define loc_tg(_y, _x)  (_x-x) ? itofix(_y-y)/(_x-x) : itofix(_y-y)

   tg_end = loc_tg(ey, ex);

   while (!done) {
      proc(bmp, px, py, d);

      /* from here, we have only 3 possible direction of movement, eg.
       * for the first quadrant:
       *
       *    OOOOOOOOO
       *    OOOOOOOOO
       *    OOOOOO21O
       *    OOOOOO3*O
       */

      /* evaluate the 3 possible points */
      switch (q) {

     case 1:
        px1 = px;
        py1 = py-1;
        px2 = px-1;
        py2 = py-1;
        px3 = px-1;
        py3 = py;

        /* 2nd quadrant check */
        if (px != x) {
           break;
        }
        else {
           /* we were in the end quadrant, changing is illegal. Exit. */
           if (qe == q)
          done = TRUE;
           q++;
        }
        /* fall through */

     case 2:
        px1 = px-1;
        py1 = py;
        px2 = px-1;
        py2 = py+1;
        px3 = px;
        py3 = py+1;

        /* 3rd quadrant check */
        if (py != y) {
           break;
        }
        else {
           /* we were in the end quadrant, changing is illegal. Exit. */
           if (qe == q)
          done = TRUE;
           q++;
        }
        /* fall through */

     case 3:
        px1 = px;
        py1 = py+1;
        px2 = px+1;
        py2 = py+1;
        px3 = px+1;
        py3 = py;

        /* 4th quadrant check */
        if (px != x) {
           break;
        }
        else {
           /* we were in the end quadrant, changing is illegal. Exit. */
           if (qe == q)
          done = TRUE;
           q++;
        }
        /* fall through */

     case 4:
        px1 = px+1;
        py1 = py;
        px2 = px+1;
        py2 = py-1;
        px3 = px;
        py3 = py-1;

        /* 1st quadrant check */
        if (py == y) {
           /* we were in the end quadrant, changing is illegal. Exit. */
           if (qe == q)
          done = TRUE;

           q = 1;
           px1 = px;
           py1 = py-1;
           px2 = px-1;
           py2 = py-1;
           px3 = px-1;
           py3 = py;
        }
        break;

     default:
        return;
      }

      /* now, we must decide which of the 3 points is the right point.
       * We evaluate the distance from center and, then, choose the
       * nearest point.
       */
      ax = x-px1;
      ay = y-py1;
      rr1 = ax*ax + ay*ay;

      ax = x-px2;
      ay = y-py2;
      rr2 = ax*ax + ay*ay;

      ax = x-px3;
      ay = y-py3;
      rr3 = ax*ax + ay*ay;

      /* difference from the main radius */
      if (rr1 > rr)
     d1 = rr1-rr;
      else
     d1 = rr-rr1;
      if (rr2 > rr)
     d2 = rr2-rr;
      else
     d2 = rr-rr2;
      if (rr3 > rr)
     d3 = rr3-rr;
      else
     d3 = rr-rr3;

      /* what is the minimum? */
      if (d1 <= d2) {
     px = px1;
     py = py1;
      }
      else if (d2 <= d3) {
     px = px2;
     py = py2;
      }
      else {
     px = px3;
     py = py3;
      }

      /* are we in the final quadrant? */
      if (qe == q) {
     tg_cur = loc_tg(py, px);

     /* is the arc finished? */
     switch (q) {

        case 1:
           /* end quadrant = 1? */
           if (tg_cur <= tg_end)
          done = TRUE;
           break;

        case 2:
           /* end quadrant = 2? */
           if (tg_cur <= tg_end)
          done = TRUE;
           break;

        case 3:
           /* end quadrant = 3? */
           if (tg_cur <= tg_end)
          done = TRUE;
           break;

        case 4:
           /* end quadrant = 4? */
           if (tg_cur <= tg_end)
          done = TRUE;
           break;
     }
      }
   }

   /* draw the last evaluated point */
   proc(bmp, px, py, d);
}



/* arc:
 *  Draws an arc.
 */
void fixed_arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int color)
{
   acquire_bitmap(bmp);

   fixed_do_arc(bmp, x, y, ang1, ang2, r, color, bmp->vtable->putpixel);

   release_bitmap(bmp);
}

#endif
