/**********
Copyright (c) 2003-2004 Paolo Fallucca.
**********/

#include "spice.h"
#include "util.h"
#include "mfb.h"
#include "cpdefs.h"
#include "ftedefs.h"
#include "ftegraph.h"
#include "ftedbgra.h"
#include "ftedev.h"

#ifdef DJGPP
#include <dos.h>
#include <io.h>
#endif

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "mfbpc.h"
#include <glib/gprintf.h>
#include <gtk/gtk.h>
#include "suffix.h"
#include "../../winres/spicep_ico.xpm"
#include "../../winres/logo.xpm"

#define redraw() gdk_gc_set_foreground(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], pal); \
                 gdk_window_invalidate_rect(drawing_area->window, NULL, TRUE)
#define blit(srcbmp,destbmp,src_x,src_y,dest_x,dest_y,width,height) \
        gdk_draw_pixmap(destbmp,drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],srcbmp,src_x,src_y,dest_x,dest_y,width,height);

#define print(S, C) {							      \
	in.h.ah = 0x05;							      \
	for (count = C, string = S; count > 0; count -= 1) {		      \
		in.h.dl = *string++;					      \
		int86(0x21, &in, &in);					      \
	}								      \
}

#define FINI_PRINTER "\033@\n\r\n"

#define IDC_RADIO_AUTORANGE FALSE
#define IDC_RADIO_USERDEFINED TRUE
#define IDC_RADIO_XLIN FALSE
#define IDC_RADIO_XLOG TRUE
#define IDC_RADIO_YLIN FALSE
#define IDC_RADIO_YLOG TRUE
#define IDC_RADIO_DEFAULT   0
#define IDC_RADIO_POLAR     1
#define IDC_RADIO_SMITH     2
#define IDC_RADIO_SMITHGRID 3

//static char *mfbpc_buffer;
int graph_mode_on = 0;
int mfbpc_width;
int mfbpc_height;

int currcolor;
unsigned short sel_style;

//static int mfbpc_double;

MFB *MFBCurrent;

struct videoconfig config;

static unsigned short line_styles[ ] = {
    0xffff, 0xaaaa, 0x7777, 0xcccc,
    0xf0f0, 0xfc10, 0x7575, 0x3f11,
    0x4444 };

unsigned char c_map[95][7] = {
    { 0x20, 0x20, 0x20, 0x20, 0x20, 0x00, 0x20 },
    { 0x50, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00 },
    { 0x00, 0x50, 0xf8, 0x50, 0x50, 0xf8, 0x50 },
    { 0x70, 0xa8, 0xa0, 0x70, 0x28, 0xa8, 0x70 },
    { 0xf8, 0xc8, 0x10, 0x20, 0x40, 0x98, 0x98 },
    { 0x40, 0xa0, 0xa0, 0x40, 0xa8, 0x90, 0x68 },
    { 0x60, 0x60, 0xc0, 0x00, 0x00, 0x00, 0x00 },
    { 0x08, 0x10, 0x20, 0x20, 0x20, 0x10, 0x08 },
    { 0x80, 0x40, 0x20, 0x20, 0x20, 0x40, 0x80 },
    { 0x20, 0xa8, 0x70, 0x20, 0x70, 0xa8, 0x20 },
    { 0x00, 0x20, 0x20, 0xf8, 0x20, 0x20, 0x00 },
    { 0x00, 0x00, 0x00, 0x00, 0x60, 0x60, 0xc0 },
    { 0x00, 0x00, 0x00, 0xf8, 0x00, 0x00, 0x00 },
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x60 },
    { 0x00, 0x08, 0x10, 0x20, 0x40, 0x80, 0x00 },
    { 0x70, 0x88, 0x98, 0xa8, 0xc8, 0x88, 0x70 },
    { 0x20, 0x60, 0x20, 0x20, 0x20, 0x20, 0x70 },
    { 0x70, 0x88, 0x08, 0x10, 0x20, 0x40, 0xf8 },
    { 0x70, 0x88, 0x08, 0x30, 0x08, 0x88, 0x70 },
    { 0x10, 0x30, 0x50, 0x90, 0xf8, 0x10, 0x10 },
    { 0xf8, 0x80, 0xf0, 0x08, 0x08, 0x88, 0x70 },
    { 0x30, 0x40, 0x80, 0xf0, 0x88, 0x88, 0x70 },
    { 0xf8, 0x08, 0x10, 0x10, 0x20, 0x20, 0x20 },
    { 0x70, 0x88, 0x88, 0x70, 0x88, 0x88, 0x70 },
    { 0x70, 0x88, 0x88, 0x78, 0x08, 0x10, 0x60 },
    { 0x00, 0x00, 0x60, 0x60, 0x00, 0x60, 0x60 },
    { 0x00, 0x00, 0x60, 0x60, 0x00, 0x60, 0x60 },
    { 0x00, 0x10, 0x20, 0x40, 0x20, 0x10, 0x00 },
    { 0x00, 0x00, 0xf8, 0x00, 0xf8, 0x00, 0x00 },
    { 0x00, 0x40, 0x20, 0x10, 0x20, 0x40, 0x00 },
    { 0x70, 0x88, 0x10, 0x20, 0x20, 0x00, 0x20 },
    { 0x70, 0x88, 0xb8, 0xa8, 0xb8, 0x80, 0x70 },
    { 0x70, 0x88, 0x88, 0xf8, 0x88, 0x88, 0x88 },
    { 0xf0, 0x88, 0x88, 0xf0, 0x88, 0x88, 0xf0 },
    { 0x70, 0x88, 0x80, 0x80, 0x80, 0x88, 0x70 },
    { 0xf0, 0x48, 0x48, 0x48, 0x48, 0x48, 0xf0 },
    { 0xf8, 0x80, 0x80, 0xf0, 0x80, 0x80, 0xf8 },
    { 0xf8, 0x80, 0x80, 0xf0, 0x80, 0x80, 0x80 },
    { 0x70, 0x88, 0x80, 0x80, 0x98, 0x88, 0x70 },
    { 0x88, 0x88, 0x88, 0xf8, 0x88, 0x88, 0x88 },
    { 0x70, 0x20, 0x20, 0x20, 0x20, 0x20, 0x70 },
    { 0x08, 0x08, 0x08, 0x08, 0x08, 0x88, 0x70 },
    { 0x88, 0x90, 0xa0, 0xc0, 0xa0, 0x90, 0x88 },
    { 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xf8 },
    { 0x88, 0xd8, 0xa8, 0x88, 0x88, 0x88, 0x88 },
    { 0x88, 0x88, 0xc8, 0xa8, 0x98, 0x88, 0x88 },
    { 0x70, 0x88, 0x88, 0x88, 0x88, 0x88, 0x70 },
    { 0xf0, 0x88, 0x88, 0xf0, 0x80, 0x80, 0x80 },
    { 0x70, 0x88, 0x88, 0x88, 0xa8, 0x90, 0x68 },
    { 0xf0, 0x88, 0x88, 0xf0, 0xa0, 0x90, 0x88 },
    { 0x70, 0x88, 0x80, 0x70, 0x08, 0x88, 0x70 },
    { 0xf8, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 },
    { 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x70 },
    { 0x88, 0x88, 0x88, 0x88, 0x50, 0x50, 0x20 },
    { 0x88, 0x88, 0x88, 0x88, 0xa8, 0xd8, 0x88 },
    { 0x88, 0x88, 0x50, 0x20, 0x50, 0x88, 0x88 },
    { 0x88, 0x88, 0x50, 0x20, 0x20, 0x20, 0x20 },
    { 0xf8, 0x08, 0x10, 0xf8, 0x40, 0x80, 0xf8 },
    { 0x38, 0x20, 0x20, 0x20, 0x20, 0x20, 0x38 },
    { 0x00, 0x80, 0x40, 0x20, 0x10, 0x08, 0x00 },
    { 0xe0, 0x20, 0x20, 0x20, 0x20, 0x20, 0xe0 },
    { 0x20, 0x50, 0x88, 0x00, 0x00, 0x00, 0x00 },
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
    { 0x30, 0x30, 0x18, 0x00, 0x00, 0x00, 0x00 },
    { 0x00, 0x00, 0x70, 0x08, 0x78, 0x88, 0x78 },
    { 0x80, 0x80, 0xf0, 0x88, 0x88, 0x88, 0xf0 },
    { 0x00, 0x00, 0x70, 0x88, 0x80, 0x80, 0x78 },
    { 0x08, 0x08, 0x78, 0x88, 0x88, 0x88, 0x78 },
    { 0x00, 0x00, 0x70, 0x88, 0xf0, 0x80, 0x70 },
    { 0x30, 0x48, 0x40, 0xe0, 0x40, 0x40, 0x40 },
    { 0x00, 0x70, 0x88, 0x88, 0x78, 0x08, 0x70 },
    { 0x80, 0x80, 0xf0, 0x88, 0x88, 0x88, 0x88 },
    { 0x20, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20 },
    { 0x08, 0x00, 0x08, 0x08, 0x08, 0x88, 0x70 },
    { 0x80, 0x80, 0x88, 0x90, 0xe0, 0x90, 0x88 },
    { 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 },
    { 0x00, 0x00, 0xd0, 0xa8, 0xa8, 0xa8, 0xa8 },
    { 0x00, 0x00, 0xb0, 0xc8, 0x88, 0x88, 0x88 },
    { 0x00, 0x00, 0x70, 0x88, 0x88, 0x88, 0x70 },
    { 0x00, 0xf0, 0x88, 0x88, 0xf0, 0x80, 0x80 },
    { 0x00, 0x70, 0x88, 0x88, 0x78, 0x08, 0x08 },
    { 0x00, 0x00, 0xb0, 0xc8, 0x80, 0x80, 0x80 },
    { 0x00, 0x00, 0x78, 0x80, 0x70, 0x08, 0xf0 },
    { 0x20, 0x20, 0xf8, 0x20, 0x20, 0x20, 0x18 },
    { 0x00, 0x00, 0x88, 0x88, 0x88, 0x88, 0x70 },
    { 0x00, 0x00, 0x88, 0x88, 0x88, 0x50, 0x20 },
    { 0x00, 0x00, 0x88, 0x88, 0xa8, 0xa8, 0x50 },
    { 0x00, 0x00, 0x88, 0x50, 0x20, 0x50, 0x88 },
    { 0x00, 0x88, 0x88, 0x50, 0x20, 0x40, 0x80 },
    { 0x00, 0x00, 0xf8, 0x10, 0x70, 0x40, 0xf8 },
    { 0x08, 0x10, 0x10, 0x20, 0x10, 0x10, 0x08 },
    { 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 },
    { 0x80, 0x40, 0x40, 0x20, 0x40, 0x40, 0x80 },
    { 0x68, 0xb0, 0x00, 0x00, 0x00, 0x00, 0x00 },
    { 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8 }
    };

static int initialized = 0;
//static short graphics_mode = 0;
//static void (interrupt far *hc_vect)( );
//static void (interrupt far mfbpc_hardcopy)( );

char AppName[]="Spice+";
char TitleName[512];
PLOT dataplot[MAXNUMPLOT];
PLOT* curr_dataplot;

GtkWidget *main_window;

int num_plot;
int curr_plot;
int menu_on = 0;
GtkWidget *drawing_area;
GdkPixmap *bmp, *plotsbmp, *bmptmp;
GdkPixmap *currbmp;
GtkWidget *vbox;
GtkWidget *menubar;
GdkColormap *colormap;
GdkColor pal[256];
GtkWidget *curs_en_dis;
GtkWidget *curlabel;
gulong curs_handler_id;
GtkWidget *curs_window;
struct st_event
  {
  int x;
  int y;
  int w;
  int h;
  int event;
  };
typedef struct st_event EVENT;
enum states {NONE, STARTSELECT};
EVENT main_event;
GtkWidget *entries[2];

gboolean iRange;
gboolean iScale;
gint iGrid;
gboolean iCurrXRange = IDC_RADIO_AUTORANGE;
gboolean iCurrXScale = IDC_RADIO_XLIN;
gboolean iCurrYRange = IDC_RADIO_AUTORANGE;
gboolean iCurrYScale = IDC_RADIO_YLIN;
gint iCurrGrid = IDC_RADIO_DEFAULT;
gchar xlo[20];
gchar xhi[20];
gchar ylo[20];
gchar yhi[20];
char c1str[40],c2str[40],difstr[40];
GtkWidget *c1str_label, *c2str_label, *difstr_label;

int menu_width = 0;
char *trace;

// Mouse Position
volatile int mouse_x;
volatile int mouse_y;
int cursor_enabled = 0;
int zoom_enabled = 0;
int createwindow;

#define NSTYLES	(NUMELEMS(line_styles))

extern void free_tracelist(wordlist **wlist);

extern int add_plot();
extern int remove_plot();
extern void select_plot_lpress();
extern wordlist *vec2wl_copy(struct dvec *vecs);

void check_param()
{
// Initialize x_axis and y_axis radio buttons
if ((plotparam.gtype == GRID_LIN) || (plotparam.gtype == GRID_YLOG))
    {
    iCurrXScale = IDC_RADIO_XLIN;
    }
else if ((plotparam.gtype == GRID_XLOG) || (plotparam.gtype == GRID_LOGLOG))
    {
    iCurrXScale = IDC_RADIO_XLOG;
    }

if ((plotparam.gtype == GRID_LIN) || (plotparam.gtype == GRID_XLOG))
    {
    iCurrYScale = IDC_RADIO_YLIN;
    }
else if ((plotparam.gtype == GRID_YLOG) || (plotparam.gtype == GRID_LOGLOG))
    {
    iCurrYScale = IDC_RADIO_YLOG;
    }

// Initialize xlo xhi ylo yhi
g_sprintf(xlo,"%g",plotparam.xlo);
g_sprintf(xhi,"%g",plotparam.xhi);
g_sprintf(ylo,"%g",plotparam.ylo);
g_sprintf(yhi,"%g",plotparam.yhi);

// Unit version (preliminary)
// require the inverse function that transform 10k->10000 for example
/*sprintf(xlo,"%g%c",plotparam.xlo,findunit(&plotparam.xlo));
sprintf(xhi,"%g%c",plotparam.xhi,findunit(&plotparam.xhi));
sprintf(ylo,"%g%c",plotparam.ylo,findunit(&plotparam.ylo));
sprintf(yhi,"%g%c",plotparam.yhi,findunit(&plotparam.yhi));*/
}

//int menu(wordlist *wl)
int menu(struct dvec *vecs)
{
wordlist *wltmp;
int i;

if (menu_on == 0)
    {
    num_plot = 1;
    curr_plot = 0;

    blit(bmp,plotsbmp,0,0,0,0,config.numxpixels,config.numypixels);
    // Reset foreground color to black (menu drawing)
    gdk_gc_set_foreground(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], pal);

    redraw();
    menu_on = 1;
    for (i=0; i<MAXNUMPLOT; i++)
        {
        dataplot[i].tracelist = NULL;
        dataplot[i].ntrace = 0;
        }

    //dataplot[0].tracelist = wl_copy(wl);
    dataplot[0].tracelist = vec2wl_copy(vecs);
    dataplot[0].graphic = CopyGraph(currentgraph);
    curr_dataplot = &dataplot[0];
    set_plot_param();
    check_param();

    for (wltmp = curr_dataplot->tracelist; wltmp; wltmp = wltmp->wl_next)
        curr_dataplot->ntrace++;

    Set_plot_rect(&(curr_dataplot->rgraph),curr_dataplot->graphic);

    // Main loop
    gtk_main();

    menu_on = 0;
    graph_mode_on = 0;
    return(0);
    }

return 0;
}

MFB *
MFBOpen(name, device, error)
    char *name, *device;
    int *error;
{
    static MFB	onlyone;
    MFB		*thismfb;

    thismfb = &onlyone;

    MFBCurrent = thismfb;

    thismfb->fileDesc = fileno(stdout);

    if (!initialized) {
        mfbpc_width  = 632;
        mfbpc_height = 453;
        config.numxpixels=mfbpc_width;
        config.numypixels=mfbpc_height;
        config.numcolors = 256;
        initialized = 1;
        gtk_disable_setlocale();
#ifdef WANT_PCHARDCOPY
	    hc_vect = _dos_getvect(5);
#endif
    }

    if (error)
        *error = MFBOK;

    return (thismfb);
}

gint on_delete_event( GtkWidget *widget,
                      GdkEvent  *event,
                      gpointer   data )
{
  if (curs_window != NULL)
    {
    if (GTK_WIDGET_VISIBLE(curs_window))
      {
      gtk_widget_destroy(curs_window);
      }
    }

  return(FALSE);
}

void
on_destroy ()
{
  gtk_main_quit();
}

void set_pal()
{
  int i;

  colormap = gdk_colormap_get_system();

  // Set palette color
  for (i=0; i<256; i++)
      gdk_color_alloc(colormap,pal+i);
}

static gint
on_configure_event (GtkWidget *widget, GdkEventConfigure *event)
{
  int resize = 0;
  int i;

  if (ft_circuits)
    if (ft_circuits->ci_name)
      sprintf(TitleName, "%s - %s", AppName, ft_circuits->ci_name);

  mfbpc_width  = widget->allocation.width;
  mfbpc_height = widget->allocation.height;
  config.numxpixels = mfbpc_width;
  config.numypixels = mfbpc_height / num_plot;
  config.numcolors  = 256;

  if (bmp)
    {
    gdk_pixmap_unref(bmp);
    gdk_pixmap_unref(bmptmp);
    gdk_pixmap_unref(plotsbmp);
    resize = 1;
    }

  bmp = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
  bmptmp = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
  plotsbmp = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
  currbmp = bmp;
  clear_to_color(currbmp,0);

  // Resize
  if (resize && (createwindow == 0))
  //if (resize)
    {
    dispdev->width = config.numxpixels + 1;
    dispdev->height = config.numypixels + 1;
    dispdev->numcolors = config.numcolors - 1;
    for (i = 0; i < num_plot; i++)
      Replot(i);
    }

  // Reset foreground color to black (menu drawing)
  gdk_gc_set_foreground(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], pal);

return TRUE;
}

static gint
on_expose_event (GtkWidget *widget, GdkEventExpose *event)
{
blit(plotsbmp, widget->window, event->area.x, event->area.y,
     event->area.x, event->area.y,event->area.width, event->area.height);
if (num_plot > 1)
    {
    MFBSetColor(5);
    gdk_draw_arc(widget->window, main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
            FALSE, curr_dataplot->rgraph.x + curr_dataplot->rgraph.w + 5,
            curr_dataplot->rgraph.y + config.numypixels * curr_plot + 
            curr_dataplot->rgraph.h / 2 - 4, 8, 8, 0, 23040);
    MFBSetColor(0);
    }

/*gdk_draw_pixmap(widget->window,
                widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                currbmp,
                event->area.x, event->area.y,
                event->area.x, event->area.y,
                event->area.width, event->area.height);*/
return FALSE;
}

int add_trace_ok()
{
wordlist *wtracel;
int ntraceadded;
struct pnode *names,*n;

// Add the new trace to the current trace list
//wl_add(&(curr_dataplot->tracelist),strlwr(trace));
//wtracel = cp_parse(strlwr(trace));
wtracel = cp_parse(trace);

names = ft_getpnames(wtracel, false);
free_tracelist(&(wtracel));
for (n = names; n; n = n->pn_next)
    {
    wl_add(&(wtracel),n->pn_name);
    }

ntraceadded = wl_length(wtracel);
curr_dataplot->ntrace += ntraceadded;
curr_dataplot->tracelist = wl_append(curr_dataplot->tracelist,wtracel);
// and plot
/*clear_to_color(bmp,0);
gr_redrawgrid(currentgraph);
blit(bmp,plotsbmp,0,0,0,config.numypixels * 0,
     config.numxpixels,config.numypixels);
redraw();*/
Replot(curr_plot);

//cursor_menu[0].flags = 0;
//trace_dialog[LIST_PROC].d1 = 0;
//curr_dataplot->ntrace++;
return 0;
}

void MessageBox(char *msg)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *dialog_action_area1;
  GtkWidget *closebutton1;
  GtkWidget *label1;

  dialog1 = gtk_dialog_new();
  gtk_window_set_title (GTK_WINDOW (dialog1), TitleName);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(main_window));
  gtk_window_set_destroy_with_parent (GTK_WINDOW(dialog1), TRUE);

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 10);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new (msg);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);

  closebutton1 = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (closebutton1);
  gtk_signal_connect_object (GTK_OBJECT (closebutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));

  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), closebutton1, GTK_RESPONSE_CLOSE);
  GTK_WIDGET_SET_FLAGS (closebutton1, GTK_CAN_DEFAULT);

  gtk_widget_show (dialog1);
}

int remove_trace_ok()
{
if (curr_dataplot->ntrace == 1)
    MessageBox("You can't delete all traces!");
else
{
// Remove the trace from trace list
wl_delete(&(curr_dataplot->tracelist),trace);
// and plot
Replot(curr_plot);

//trace_dialog[LIST_PROC].d1 = 0;
curr_dataplot->ntrace--;

// Reset cursor trace
if(cursor_enabled)
    get_cursor_init_pos();
}
return 0;
}

void add_trace_ok_exec( GtkWidget *widget,
               GObject *entry )
{
  trace = (char *) gtk_entry_get_text(GTK_ENTRY(entry));
  add_trace_ok();

  redraw();

  //g_print ("Trace = %s!\n", trace);
}

void remove_trace_ok_exec( GtkWidget *widget,
               GObject *entry )
{
  trace = (char *) gtk_entry_get_text(GTK_ENTRY(entry));
  remove_trace_ok();

  redraw();

  //g_print ("Trace = %s!\n", trace);
}

void data_range( GtkWidget *widget,
               GtkWidget **entry )
{
  iRange = gtk_toggle_button_get_active((GtkToggleButton *)widget);
  gtk_widget_set_sensitive(GTK_WIDGET(*entries), iRange);
  gtk_widget_set_sensitive(GTK_WIDGET(*(entries+1)), iRange);
  gtk_editable_set_position(GTK_EDITABLE(*entries), 4);
}

void scale( GtkWidget *widget,
               GtkWidget **entry )
{
  iScale = gtk_toggle_button_get_active((GtkToggleButton *)widget);
}

void toggle_grid_butt_1( GtkWidget *widget,
                       GtkWidget **entry )
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
    iGrid = IDC_RADIO_DEFAULT;
    }
}

void toggle_grid_butt_2( GtkWidget *widget,
                       GtkWidget **entry )
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
    iGrid = IDC_RADIO_POLAR;
    }
}

void toggle_grid_butt_3( GtkWidget *widget,
                       GtkWidget **entry )
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
    iGrid = IDC_RADIO_SMITH;
    }
}

void toggle_grid_butt_4( GtkWidget *widget,
                       GtkWidget **entry )
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
    iGrid = IDC_RADIO_SMITHGRID;
    }
}

int x_axis_set_ok()
{
int i;

set_x_limits = 0;

if ((iCurrXScale == IDC_RADIO_XLIN) && (iCurrYScale == IDC_RADIO_YLIN))
    plotparam.gtype = GRID_LIN;

if ((iCurrXScale == IDC_RADIO_XLOG) && (iCurrYScale == IDC_RADIO_YLIN))
    plotparam.gtype = GRID_XLOG;

if ((iCurrXScale == IDC_RADIO_XLIN) && (iCurrYScale == IDC_RADIO_YLOG))
    plotparam.gtype = GRID_YLOG;

if ((iCurrXScale == IDC_RADIO_XLOG) && (iCurrYScale == IDC_RADIO_YLOG))
    plotparam.gtype = GRID_LOGLOG;

if (iCurrXRange == IDC_RADIO_USERDEFINED)
    {
    //GetDlgItemText(hWnd,IDC_EDIT_XLO,xlo,sizeof(xlo));
    //GetDlgItemText(hWnd,IDC_EDIT_XHI,xhi,sizeof(xhi));
    plotparam.xlo = atof(xlo);
    plotparam.xhi = atof(xhi);
    set_x_limits = 1;
    }

for (i = 0; i < num_plot; i++)
    Replot(i);

return 0;
}

int y_axis_set_ok()
{
int i;

set_y_limits = 0;

if ((iCurrXScale == IDC_RADIO_XLIN) && (iCurrYScale == IDC_RADIO_YLIN))
    plotparam.gtype = GRID_LIN;

if ((iCurrXScale == IDC_RADIO_XLOG) && (iCurrYScale == IDC_RADIO_YLIN))
    plotparam.gtype = GRID_XLOG;

if ((iCurrXScale == IDC_RADIO_XLIN) && (iCurrYScale == IDC_RADIO_YLOG))
    plotparam.gtype = GRID_YLOG;

if ((iCurrXScale == IDC_RADIO_XLOG) && (iCurrYScale == IDC_RADIO_YLOG))
    plotparam.gtype = GRID_LOGLOG;

if (iCurrYRange == IDC_RADIO_USERDEFINED)
    {
    //GetDlgItemText(hWnd,IDC_EDIT_YLO,ylo,sizeof(ylo));
    //GetDlgItemText(hWnd,IDC_EDIT_YHI,yhi,sizeof(yhi));
    plotparam.ylo = atof(ylo);
    plotparam.yhi = atof(yhi);
    set_y_limits = 1;
    }

for (i = 0; i < num_plot; i++)
    Replot(i);

return 0;
}

void x_axis_ok( GtkWidget *widget,
               GObject *entry )
{
  G_CONST_RETURN char *pxlo, *pxhi;

  iCurrXRange = iRange;
  iCurrXScale = iScale;
  pxlo = gtk_entry_get_text(GTK_ENTRY(entries[0]));
  strcpy(xlo, (char *)pxlo);
  pxhi = gtk_entry_get_text(GTK_ENTRY(entries[1]));
  strcpy(xhi, (char *)pxhi);
  //g_print ("xlo = %s!\n", xlo);
  //g_print ("xhi = %s!\n", xhi);
  x_axis_set_ok();
  redraw();
}

void y_axis_ok( GtkWidget *widget,
               GObject *entry )
{
  G_CONST_RETURN char *pylo, *pyhi;

  iCurrYRange = iRange;
  iCurrYScale = iScale;
  pylo = gtk_entry_get_text(GTK_ENTRY(entries[0]));
  strcpy(ylo, (char *)pylo);
  pyhi = gtk_entry_get_text(GTK_ENTRY(entries[1]));
  strcpy(yhi, (char *)pyhi);
  //g_print ("ylo = %s!\n", ylo);
  //g_print ("yhi = %s!\n", yhi);
  y_axis_set_ok();
  redraw();
}

void grid_ok( GtkWidget *widget,
               GObject *entry )
{
  int i;

  iCurrGrid = iGrid;
  if (iCurrGrid == IDC_RADIO_DEFAULT)
      plotparam.gtype = GRID_NONE;
  if (iCurrGrid == IDC_RADIO_POLAR)
      plotparam.gtype = GRID_POLAR;
  if (iCurrGrid == IDC_RADIO_SMITH)
      plotparam.gtype = GRID_SMITH;
  if (iCurrGrid == IDC_RADIO_SMITHGRID)
      plotparam.gtype = GRID_SMITHGRID;
  for (i = 0; i < num_plot; i++)
      Replot(i);
  redraw();
}

void on_selection_changed( GtkWidget *treeview,
                           gpointer   entry )
{
  GtkTreeSelection *treesel;
  GtkTreeIter iter;
  GtkSelectionMode selmode;
  GtkTreeModel *model;
  gchar *str_data;

  treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
  selmode = gtk_tree_selection_get_mode(treesel);
  gtk_tree_selection_get_selected (treesel, NULL, &iter);
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
  gtk_tree_model_get (GTK_TREE_MODEL(model), &iter,
                          0, &str_data,
                          -1);

  gtk_entry_set_text(GTK_ENTRY(entry), str_data);
}

static void x_axis_dialog( GtkWidget *w,
                     gpointer   data )
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *vbox2;
  GtkWidget *vbox3;

  GtkWidget *frame1;
  GtkWidget *label1;
  GSList    *radiobutton1_group = NULL;
  GtkWidget *radiobutton1;
  GtkWidget *radiobutton2;
  GtkWidget *hbox1;
  GtkWidget *hbox2;
  GtkWidget *entry1;
  GtkWidget *entry2;

  GtkWidget *frame2;
  GtkWidget *label2;
  GSList    *radiobutton2_group = NULL;
  GtkWidget *radiobutton3;
  GtkWidget *radiobutton4;

  GtkWidget *dialog_action_area1;
  GtkWidget *cancelbutton1;
  GtkWidget *okbutton1;

  int textpos;
  PangoLayout *p_layout;
  PangoAttrList *p_attr_list;

  dialog1 = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (dialog1), 6);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(main_window));
  gtk_window_set_destroy_with_parent (GTK_WINDOW(dialog1), TRUE);
  gtk_window_set_title (GTK_WINDOW (dialog1), "X Axis Setting");

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 4);
  gtk_frame_set_label_align (GTK_FRAME (frame1), 0.03, 0.5);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame1), vbox2);

  label1 = gtk_label_new ("Data Range");
  gtk_widget_show (label1);
  gtk_frame_set_label_widget (GTK_FRAME (frame1), label1);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

  radiobutton1 = gtk_radio_button_new_with_mnemonic (NULL, "Auto Range");
  gtk_widget_show (radiobutton1);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton1, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton1), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton1));

  radiobutton2 = gtk_radio_button_new_with_mnemonic (NULL, "User Defined");
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton2, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton2), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton2));
  iRange = iCurrXRange;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton2), iRange);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (hbox1), hbox2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox2), 6);

  entry1 = gtk_entry_new ();
  gtk_widget_set_sensitive(entry1, iRange);
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox2), entry1, TRUE, TRUE, 6);
  gtk_editable_insert_text(GTK_EDITABLE(entry1), xlo, strlen(xlo), &textpos);
  //p_layout = gtk_widget_create_pango_layout(entry1, "Pippo");
  p_layout = gtk_entry_get_layout(GTK_ENTRY(entry1));

  pango_layout_set_alignment(p_layout, PANGO_ALIGN_RIGHT);
  p_attr_list = pango_layout_get_attributes(p_layout);

  //gtk_widget_modify_base(entry1, GTK_STATE_NORMAL, pal+2);
  //gtk_widget_modify_text(entry1, GTK_STATE_NORMAL, pal+3);
  //gtk_widget_set_direction(entry1, GTK_TEXT_DIR_RTL);
  //gtk_editable_set_position(GTK_EDITABLE(entry1), 4);

  entry2 = gtk_entry_new ();
  gtk_widget_set_sensitive(entry2, iRange);
  gtk_widget_show (entry2);
  gtk_box_pack_start (GTK_BOX (hbox2), entry2, TRUE, TRUE, 6);
  gtk_editable_insert_text(GTK_EDITABLE(entry2), xhi, strlen(xhi), &textpos);

  entries[0] = entry1;
  entries[1] = entry2;

  frame2 = gtk_frame_new (NULL);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 4);
  gtk_frame_set_label_align (GTK_FRAME (frame2), 0.03, 0.5);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (frame2), vbox3);

  label2 = gtk_label_new ("Scale");
  gtk_widget_show (label2);
  gtk_frame_set_label_widget (GTK_FRAME (frame2), label2);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

  radiobutton3 = gtk_radio_button_new_with_mnemonic (NULL, "Linear");
  gtk_widget_show (radiobutton3);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton3, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton3), radiobutton2_group);
  radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton3));

  radiobutton4 = gtk_radio_button_new_with_mnemonic (NULL, "Log");
  gtk_widget_show (radiobutton4);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton4, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton4), radiobutton2_group);
  radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton4));
  iScale = iCurrXScale;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton4), iScale);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_signal_connect_object (GTK_OBJECT (cancelbutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  g_signal_connect (G_OBJECT (okbutton1), "clicked",
		    G_CALLBACK (x_axis_ok), NULL);
  gtk_signal_connect_object (GTK_OBJECT (okbutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect (G_OBJECT (radiobutton2), "toggled",
		    G_CALLBACK (data_range), entries);
  g_signal_connect (G_OBJECT (radiobutton4), "toggled",
		    G_CALLBACK (scale), entries);
  /*g_signal_connect (G_OBJECT (entry1), "style_set",
		    G_CALLBACK (pango_layout_context_changed), p_layout);*/

  gtk_window_set_position(GTK_WINDOW(dialog1), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_widget_show_all (dialog1);
}

static void y_axis_dialog( GtkWidget *w,
                     gpointer   data )
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *vbox2;
  GtkWidget *vbox3;

  GtkWidget *frame1;
  GtkWidget *label1;
  GSList    *radiobutton1_group = NULL;
  GtkWidget *radiobutton1;
  GtkWidget *radiobutton2;
  GtkWidget *hbox1;
  GtkWidget *hbox2;
  GtkWidget *entry1;
  GtkWidget *entry2;

  GtkWidget *frame2;
  GtkWidget *label2;
  GSList    *radiobutton2_group = NULL;
  GtkWidget *radiobutton3;
  GtkWidget *radiobutton4;

  GtkWidget *dialog_action_area1;
  GtkWidget *cancelbutton1;
  GtkWidget *okbutton1;

  int textpos;

  dialog1 = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (dialog1), 6);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(main_window));
  gtk_window_set_title (GTK_WINDOW (dialog1), "Y Axis Setting");

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 4);
  gtk_frame_set_label_align (GTK_FRAME (frame1), 0.03, 0.5);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame1), vbox2);

  label1 = gtk_label_new ("Data Range");
  gtk_widget_show (label1);
  gtk_frame_set_label_widget (GTK_FRAME (frame1), label1);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

  radiobutton1 = gtk_radio_button_new_with_mnemonic (NULL, "Auto Range");
  gtk_widget_show (radiobutton1);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton1, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton1), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton1));

  radiobutton2 = gtk_radio_button_new_with_mnemonic (NULL, "User Defined");
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton2, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton2), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton2));
  iRange = iCurrYRange;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton2), iRange);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (hbox1), hbox2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox2), 6);

  entry1 = gtk_entry_new ();
  gtk_widget_set_sensitive(entry1, iRange);
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox2), entry1, TRUE, TRUE, 6);
  gtk_editable_insert_text(GTK_EDITABLE(entry1), ylo, strlen(ylo), &textpos);

  entry2 = gtk_entry_new ();
  gtk_widget_set_sensitive(entry2, iRange);
  gtk_widget_show (entry2);
  gtk_box_pack_start (GTK_BOX (hbox2), entry2, TRUE, TRUE, 6);
  gtk_editable_insert_text(GTK_EDITABLE(entry2), yhi, strlen(yhi), &textpos);

  entries[0] = entry1;
  entries[1] = entry2;

  frame2 = gtk_frame_new (NULL);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 4);
  gtk_frame_set_label_align (GTK_FRAME (frame2), 0.03, 0.5);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (frame2), vbox3);

  label2 = gtk_label_new ("Scale");
  gtk_widget_show (label2);
  gtk_frame_set_label_widget (GTK_FRAME (frame2), label2);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

  radiobutton3 = gtk_radio_button_new_with_mnemonic (NULL, "Linear");
  gtk_widget_show (radiobutton3);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton3, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton3), radiobutton2_group);
  radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton3));

  radiobutton4 = gtk_radio_button_new_with_mnemonic (NULL, "Log");
  gtk_widget_show (radiobutton4);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton4, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton4), radiobutton2_group);
  radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton4));
  iScale = iCurrYScale;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton4), iScale);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_signal_connect_object (GTK_OBJECT (cancelbutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  g_signal_connect (G_OBJECT (okbutton1), "clicked",
		    G_CALLBACK (y_axis_ok), NULL);
  gtk_signal_connect_object (GTK_OBJECT (okbutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect (G_OBJECT (radiobutton2), "toggled",
		    G_CALLBACK (data_range), entries);
  g_signal_connect (G_OBJECT (radiobutton4), "toggled",
		    G_CALLBACK (scale), entries);

  gtk_window_set_position(GTK_WINDOW(dialog1), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_widget_show_all (dialog1);
}

static void grid_dialog( GtkWidget *w,
                     gpointer   data )
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *vbox2;

  GtkWidget *frame1;
  GtkWidget *label1;
  GSList    *radiobutton1_group = NULL;
  GtkWidget *radiobutton1;
  GtkWidget *radiobutton2;
  GtkWidget *radiobutton3;
  GtkWidget *radiobutton4;

  GtkWidget *dialog_action_area1;
  GtkWidget *cancelbutton1;
  GtkWidget *okbutton1;

  dialog1 = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (dialog1), 6);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(main_window));
  gtk_window_set_title (GTK_WINDOW (dialog1), "Grid");

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 4);
  gtk_frame_set_label_align (GTK_FRAME (frame1), 0.03, 0.5);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame1), vbox2);

  label1 = gtk_label_new ("Grid Type");
  gtk_widget_show (label1);
  gtk_frame_set_label_widget (GTK_FRAME (frame1), label1);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

  radiobutton1 = gtk_radio_button_new_with_mnemonic (NULL, "Default");
  gtk_widget_show (radiobutton1);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton1, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton1), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton1));

  radiobutton2 = gtk_radio_button_new_with_mnemonic (NULL, "Polar");
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton2, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton2), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton2));

  radiobutton3 = gtk_radio_button_new_with_mnemonic (NULL, "Smith");
  gtk_widget_show (radiobutton3);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton3, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton3), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton3));

  radiobutton4 = gtk_radio_button_new_with_mnemonic (NULL, "SmithGrid");
  gtk_widget_show (radiobutton4);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton4, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton4), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton4));

  iGrid = iCurrGrid;
  switch (iGrid)
      {
      case 0:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton1), TRUE);
      break;

      case 1:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton2), TRUE);
      break;

      case 2:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton3), TRUE);
      break;

      case 3:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton4), TRUE);
      break;
      }

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_signal_connect_object (GTK_OBJECT (cancelbutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  g_signal_connect (G_OBJECT (okbutton1), "clicked",
		    G_CALLBACK (grid_ok), NULL);
  gtk_signal_connect_object (GTK_OBJECT (okbutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect (G_OBJECT (radiobutton1), "toggled",
		    G_CALLBACK (toggle_grid_butt_1), NULL);
  g_signal_connect (G_OBJECT (radiobutton2), "toggled",
		    G_CALLBACK (toggle_grid_butt_2), NULL);
  g_signal_connect (G_OBJECT (radiobutton3), "toggled",
		    G_CALLBACK (toggle_grid_butt_3), NULL);
  g_signal_connect (G_OBJECT (radiobutton4), "toggled",
		    G_CALLBACK (toggle_grid_butt_4), NULL);

  gtk_window_set_position(GTK_WINDOW(dialog1), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_widget_show_all (dialog1);
}

static void trace_add_dialog( GtkWidget *w,
                     gpointer   data )
{
  GtkWidget *window;
  GtkWidget *vbox, *hbox;
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *scrolled_window;
  GtkWidget *tree_view;
  GtkCellRenderer *cell;
  GtkTreeViewColumn *column;
  GtkTreeIter iter;
  GtkListStore *liststore;
  GtkWidget *entry;
  struct dvec *v;
  GtkWidget *hbuttonbox;

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(main_window));
  gtk_window_set_title (GTK_WINDOW(window), "Add Trace...");
  gtk_container_set_border_width (GTK_CONTAINER (window), 10);
  vbox = gtk_vbox_new (FALSE, 0);

  // Trace label
  hbox = gtk_hbox_new (FALSE, 10);
  label = gtk_label_new ("Trace");
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);
  entry = gtk_entry_new();
  //gtk_editable_select_region (GTK_EDITABLE (entry), 0, 5);
  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  // Trace list (ListStore)
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
  liststore = gtk_list_store_new(1, G_TYPE_STRING);
  tree_view = gtk_tree_view_new();

  /*treestore = gtk_tree_store_new(1, G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF,
               G_TYPE_FLOAT, G_TYPE_UINT, G_TYPE_UCHAR, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_INT);
  tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL (treestore));*/

  //gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), tree_view);
  gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
  gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
  gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), GTK_TREE_MODEL (liststore));
  gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (tree_view);

  // Add available vectors to list
  cp_periodic();
  for (v = plot_cur->pl_dvecs; v; v = v->v_next)
    {
    gtk_list_store_append (liststore, &iter);
    gtk_list_store_set (liststore, &iter, 0, v->v_name, -1);
    }

  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), FALSE);
  //gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
  cell = gtk_cell_renderer_text_new ();
  /*column = gtk_tree_view_column_new_with_attributes ("Messages",
                                                       cell,
                                                       "text", 0,
                                                       NULL);*/
  column = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title(column, "Available Traces");
  gtk_tree_view_column_pack_start(column, cell, FALSE);
  gtk_tree_view_column_set_attributes(column, cell, "text", 0, NULL);
  gtk_tree_view_column_set_clickable(column, TRUE);

  /*gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
                               GTK_TREE_VIEW_COLUMN (column));*/
  gtk_tree_view_insert_column (GTK_TREE_VIEW (tree_view), column, 0);

  g_signal_connect (G_OBJECT (tree_view), "cursor_changed",
                    G_CALLBACK (on_selection_changed), entry);

  gtk_widget_show (scrolled_window);

  // HButtonBox
  hbuttonbox = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbuttonbox, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_SPREAD);

  // Cancel button
  button = gtk_button_new_from_stock ("gtk-cancel");
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (window));
  gtk_box_pack_start (GTK_BOX (hbuttonbox), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  // OK button
  button = gtk_button_new_from_stock ("gtk-ok");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (add_trace_ok_exec), entry);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (window));
  gtk_box_pack_start (GTK_BOX (hbuttonbox), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_widget_show (vbox);
  //gtk_widget_show (window);
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_widget_show_all (window);
}

static void trace_remove_dialog( GtkWidget *w,
                     gpointer   data )
{
  GtkWidget *window;
  GtkWidget *vbox, *hbox;
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *scrolled_window;
  GtkWidget *tree_view;
  GtkCellRenderer *cell;
  GtkTreeViewColumn *column;
  GtkTreeIter iter;
  GtkListStore *liststore;
  GtkWidget *entry;
  //struct dvec *v;
  wordlist *wl;
  GtkWidget *hbuttonbox;

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(main_window));
  gtk_window_set_title (GTK_WINDOW(window), "Remove Trace...");
  gtk_container_set_border_width (GTK_CONTAINER (window), 10);
  vbox = gtk_vbox_new (FALSE, 0);

  // Trace label
  hbox = gtk_hbox_new (FALSE, 10);
  label = gtk_label_new ("Trace");
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);
  entry = gtk_entry_new();
  //gtk_editable_select_region (GTK_EDITABLE (entry), 0, 5);
  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  // Trace list (ListStore)
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
  liststore = gtk_list_store_new(1, G_TYPE_STRING);
  tree_view = gtk_tree_view_new();

  /*treestore = gtk_tree_store_new(1, G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF,
               G_TYPE_FLOAT, G_TYPE_UINT, G_TYPE_UCHAR, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_INT);
  tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL (treestore));*/

  //gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), tree_view);
  gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
  gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
  gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), GTK_TREE_MODEL (liststore));
  gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (tree_view);

  // Add available vectors to list
  /*cp_periodic();
  for (v = plot_cur->pl_dvecs; v; v = v->v_next)
    {
    gtk_list_store_append (liststore, &iter);
    gtk_list_store_set (liststore, &iter, 0, v->v_name, -1);
    }*/
  for (wl = curr_dataplot->tracelist; wl; wl = wl->wl_next)
    {
    gtk_list_store_append (liststore, &iter);
    gtk_list_store_set (liststore, &iter, 0, wl->wl_word, -1);
    }

  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), FALSE);
  //gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
  cell = gtk_cell_renderer_text_new ();
  /*column = gtk_tree_view_column_new_with_attributes ("Messages",
                                                       cell,
                                                       "text", 0,
                                                       NULL);*/
  column = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title(column, "Available Traces");
  gtk_tree_view_column_pack_start(column, cell, FALSE);
  gtk_tree_view_column_set_attributes(column, cell, "text", 0, NULL);
  gtk_tree_view_column_set_clickable(column, TRUE);

  /*gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
                               GTK_TREE_VIEW_COLUMN (column));*/
  gtk_tree_view_insert_column (GTK_TREE_VIEW (tree_view), column, 0);

  g_signal_connect (G_OBJECT (tree_view), "cursor_changed",
                    G_CALLBACK (on_selection_changed), entry);

  gtk_widget_show (scrolled_window);

  // HButtonBox
  hbuttonbox = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbuttonbox, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_SPREAD);

  // Cancel button
  button = gtk_button_new_from_stock ("gtk-cancel");
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (window));
  gtk_box_pack_start (GTK_BOX (hbuttonbox), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  // OK button
  button = gtk_button_new_from_stock ("gtk-ok");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (remove_trace_ok_exec), entry);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (window));
  gtk_box_pack_start (GTK_BOX (hbuttonbox), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_widget_show (vbox);
  //gtk_widget_show (window);
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_widget_show_all (window);
}

static void zoom_normal( GtkWidget *w,
                     gpointer   data )
{
  set_x_limits = 0;
  set_y_limits = 0;
  Replot(curr_plot);
  redraw();
}

static void zoom_area( GtkWidget *w,
                     gpointer   data )
{
  //GdkCursor *chcurs;

  //chcurs = gdk_cursor_new(GDK_CROSSHAIR);
  //gdk_window_set_cursor(main_window->window, chcurs);
  zoom_enabled = 1;
}

static void menu_quit( GtkWidget *w,
                     gpointer   data )
{
gtk_widget_destroy(data);
}

static void cursor_enable( GtkWidget *w,
                     gpointer   data );

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",               NULL,         NULL, 0, "<Branch>" },
  { "/File/Quit",           "<control>Q", menu_quit, 0, NULL },

  { "/_Trace",              NULL,         NULL, 0, "<Branch>" },
  { "/Trace/Add...",        NULL,         trace_add_dialog, 0, NULL },
  { "/Trace/Remove...",     NULL,         trace_remove_dialog, 0, NULL },

  { "/_Plot",               NULL,         NULL, 0, "<Branch>" },
  { "/Plot/X Axis...",      NULL,         x_axis_dialog, 0, NULL },
  { "/Plot/Y Axis...",      NULL,         NULL, 0, NULL },
  { "/Plot/Grid...",        NULL,         NULL, 0, NULL },
  { "/Plot/sep1",           NULL,         NULL, 0, "<Separator>" },
  { "/Plot/Add Plot",       NULL,         NULL, 0, NULL },
  { "/Plot/Remove Plot",    NULL,         NULL, 0, NULL },

  { "/_Zoom",               NULL,         NULL, 0, "<Branch>" },
  { "/Zoom/Normal",         NULL,         zoom_normal, 0, NULL },
  { "/Zoom/In",             NULL,         NULL, 0, NULL },
  { "/Zoom/Out",            NULL,         NULL, 0, NULL },
  { "/Zoom/Area",           NULL,         zoom_area, 0, NULL },

  { "/_Cursor",             NULL,         NULL, 0, "<Branch>" },
  { "/Cursor/Enable",       NULL,         cursor_enable, 0, NULL },

  { "/_Help",               NULL,         NULL, 0, "<LastBranch>" },
  { "/_Help/About",         NULL,         NULL, 0, NULL },
};

static gint cursor_disable( GtkWidget *widget,
                     gpointer data)
{
  cursor_enabled = 0;
  gtk_label_set_text(GTK_LABEL(curlabel), "Enable");
  g_signal_handler_disconnect((gpointer) curs_en_dis, curs_handler_id);
  curs_handler_id = g_signal_connect ((gpointer) curs_en_dis, "activate",
                    G_CALLBACK (cursor_enable),
                    NULL);
  gtk_widget_destroy(curs_window);
  curs_window = NULL;
  blit(bmp,plotsbmp,0,0,0,config.numypixels * curr_plot,
     config.numxpixels,config.numypixels);
  redraw();

  return TRUE;
}

static void cursor_enable( GtkWidget *widget,
                     gpointer data)
{
  GtkWidget *vbox;

  cursor_enabled = 1;
  gtk_label_set_text(GTK_LABEL(curlabel), "Disable");
  g_signal_handler_disconnect(widget, curs_handler_id);
  curs_handler_id = g_signal_connect ((gpointer) curs_en_dis, "activate",
                    G_CALLBACK (cursor_disable), NULL);

  // Create a new window
  curs_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_transient_for (GTK_WINDOW(curs_window), GTK_WINDOW(main_window));
  gtk_window_set_destroy_with_parent (GTK_WINDOW(curs_window), TRUE);
  g_signal_connect (G_OBJECT (curs_window), "delete_event",
                    G_CALLBACK (cursor_disable), NULL);

  // Create a VBox
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (curs_window), vbox);
  gtk_widget_show (vbox);

  // Find cursors positions
  plot_curs_pos(c1str, c2str, difstr);

  // c1str Label
  c1str_label = gtk_label_new (c1str);
  gtk_misc_set_alignment (GTK_MISC (c1str_label), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (vbox), c1str_label, FALSE, FALSE, 0);
  gtk_widget_show (c1str_label);

  // c2str Label
  c2str_label = gtk_label_new (c2str);
  gtk_misc_set_alignment (GTK_MISC (c2str_label), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (vbox), c2str_label, FALSE, FALSE, 0);
  gtk_widget_show (c2str_label);

  // difstr Label
  difstr_label = gtk_label_new (difstr);
  gtk_misc_set_alignment (GTK_MISC (difstr_label), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (vbox), difstr_label, FALSE, FALSE, 0);
  gtk_widget_show (difstr_label);

  // Show the window
  gtk_widget_show (curs_window);

  get_cursor_init_pos();
  draw_cursor();
  // Reset foreground color to black (menu drawing)
  //gdk_gc_set_foreground(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], pal);
  redraw();
}

void get_main_menu( GtkWidget  *window,
                    GtkWidget **menubar )
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  accel_group = gtk_accel_group_new ();

  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
              or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
              the accelerator table while generating menus.
  */

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
				       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  if (menubar)
    /* Finally, return the actual menu bar created by the item factory. */
    *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

GdkColor *rgb(guint16 red, guint16 green, guint16 blue)
{
GdkColor *color;

color = malloc(sizeof(GdkColor));
color->red = red*65535/255;
color->green = green*65535/255;
color->blue = blue*65535/255;
return color;
}

static void about_dialog( GtkWidget *widget)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *image1;
  GdkPixmap *pixmap;
  GtkStyle *style;
  GdkBitmap *mask = NULL;
  GtkWidget *dialog_action_area1;
  GtkWidget *closebutton1;
  GtkWidget *label1;
  GtkWidget *label2;
  char ProgramStr[256];
  char CopyrightStr[] = COPYRIGHT_STRING;
  //GtkWidget *hbuttonbox;

  //printf("GTK+ version = %d.%d.%d\n", gtk_major_version, gtk_minor_version, gtk_micro_version);
  dialog1 = gtk_dialog_new();
  gtk_window_set_title (GTK_WINDOW (dialog1), "About...");
  gtk_widget_modify_bg(dialog1, GTK_STATE_NORMAL, pal);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(main_window));
  gtk_window_set_destroy_with_parent (GTK_WINDOW(dialog1), TRUE);

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 10);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  style = gtk_widget_get_style(main_window);
  pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask, pal, (gchar **) logo_xpm);

  image1 = gtk_image_new_from_pixmap(pixmap, mask);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (vbox1), image1, TRUE, TRUE, 0);

  sprintf(ProgramStr,"%s %s",ft_sim->simulator, ft_sim->version);
  label1 = gtk_label_new (ProgramStr);
  gtk_widget_modify_fg(label1, GTK_STATE_NORMAL, rgb(200,200,50));
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

  label2 = gtk_label_new (CopyrightStr);
  gtk_widget_modify_fg(label2, GTK_STATE_NORMAL, rgb(200,200,50));
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_show (dialog_action_area1);
  //gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);

  closebutton1 = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (closebutton1);
  //gtk_widget_modify_bg(closebutton1, GTK_STATE_NORMAL, pal+3);
  //gtk_widget_modify_fg(closebutton1, GTK_STATE_NORMAL, pal+2);
  //gtk_widget_modify_base(closebutton1, GTK_STATE_NORMAL, pal+3);
  //gtk_widget_modify_text(closebutton1, GTK_STATE_NORMAL, pal+4);
  gtk_signal_connect_object (GTK_OBJECT (closebutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));

  // HButtonBox
  /*hbuttonbox = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox);
  gtk_container_set_border_width (GTK_CONTAINER (hbuttonbox), 10);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox, TRUE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_SPREAD);*/

  // Close button
  /*closebutton1 = gtk_button_new_from_stock ("gtk-close");
  gtk_signal_connect_object (GTK_OBJECT (closebutton1), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog1));
  gtk_box_pack_start (GTK_BOX (hbuttonbox), closebutton1, TRUE, TRUE, 0);
  gtk_widget_show (closebutton1);*/

  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), closebutton1, GTK_RESPONSE_CLOSE);
  GTK_WIDGET_SET_FLAGS (closebutton1, GTK_CAN_DEFAULT);

  gtk_widget_show (dialog1);
}

void get_main_menu2( GtkWidget  *window,
                    GtkWidget **menubar )
{
  //GtkAccelGroup *accel_group;

  GtkWidget *menuitem1;
  GtkWidget *menu1;
  GtkWidget *quit;
  GtkWidget *menuitem2;
  GtkWidget *menu2;
  GtkWidget *add;
  GtkWidget *remove;
  GtkWidget *menuitem3;
  GtkWidget *menu3;
  GtkWidget *xaxis;
  GtkWidget *yaxis;
  GtkWidget *grid;
  GtkWidget *addplot;
  GtkWidget *remplot;
  GtkWidget *menuitem4;
  GtkWidget *menu4;
  GtkWidget *zoomnormal;
  GtkWidget *zoomin;
  GtkWidget *zoomout;
  GtkWidget *zoomarea;
  GtkWidget *menuitem5;
  GtkWidget *menu5;
  GtkWidget *menuitem6;
  GtkWidget *menu6;
  GtkWidget *about;

  //accel_group = gtk_accel_group_new ();

  // Create a menu-bar to hold the menus
  *menubar = gtk_menu_bar_new ();
  gtk_widget_show (*menubar);

  // Create the "File" root menu, the label displayed on the menu bar
  menuitem1 = gtk_menu_item_new_with_mnemonic ("_File");
  gtk_widget_show (menuitem1);
  gtk_container_add (GTK_CONTAINER (*menubar), menuitem1);

  menu1 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem1), menu1);

  quit = gtk_menu_item_new_with_mnemonic ("_Quit");
  gtk_widget_show (quit);
  gtk_container_add (GTK_CONTAINER (menu1), quit);

  // Create the "Trace" root menu, the label displayed on the menu bar
  menuitem2 = gtk_menu_item_new_with_mnemonic ("_Trace");
  gtk_widget_show (menuitem2);
  gtk_container_add (GTK_CONTAINER (*menubar), menuitem2);

  menu2 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem2), menu2);

  add = gtk_menu_item_new_with_mnemonic ("_Add...");
  gtk_widget_show (add);
  gtk_container_add (GTK_CONTAINER (menu2), add);

  remove = gtk_menu_item_new_with_mnemonic ("_Remove...");
  gtk_widget_show (remove);
  gtk_container_add (GTK_CONTAINER (menu2), remove);

  // Create the "Plot" root menu, the label displayed on the menu bar
  menuitem3 = gtk_menu_item_new_with_mnemonic ("_Plot");
  gtk_widget_show (menuitem3);
  gtk_container_add (GTK_CONTAINER (*menubar), menuitem3);

  menu3 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem3), menu3);

  xaxis = gtk_menu_item_new_with_mnemonic ("_X Axis...");
  gtk_widget_show (xaxis);
  gtk_container_add (GTK_CONTAINER (menu3), xaxis);

  yaxis = gtk_menu_item_new_with_mnemonic ("_Y Axis...");
  gtk_widget_show (yaxis);
  gtk_container_add (GTK_CONTAINER (menu3), yaxis);

  grid = gtk_menu_item_new_with_mnemonic ("_Grid...");
  gtk_widget_show (grid);
  gtk_container_add (GTK_CONTAINER (menu3), grid);

  addplot = gtk_menu_item_new_with_mnemonic ("_Add Plot");
  gtk_widget_show (addplot);
  gtk_container_add (GTK_CONTAINER (menu3), addplot);

  remplot = gtk_menu_item_new_with_mnemonic ("_Remove Plot");
  gtk_widget_show (remplot);
  gtk_container_add (GTK_CONTAINER (menu3), remplot);

  // Create the "Zoom" root menu, the label displayed on the menu bar
  menuitem4 = gtk_menu_item_new_with_mnemonic ("_Zoom");
  gtk_widget_show (menuitem4);
  gtk_container_add (GTK_CONTAINER (*menubar), menuitem4);

  menu4 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem4), menu4);

  zoomnormal = gtk_menu_item_new_with_mnemonic ("_Normal");
  gtk_widget_show (zoomnormal);
  gtk_container_add (GTK_CONTAINER (menu4), zoomnormal);

  zoomin = gtk_menu_item_new_with_mnemonic ("_In");
  gtk_widget_show (zoomin);
  gtk_container_add (GTK_CONTAINER (menu4), zoomin);

  zoomout = gtk_menu_item_new_with_mnemonic ("_Out");
  gtk_widget_show (zoomout);
  gtk_container_add (GTK_CONTAINER (menu4), zoomout);

  zoomarea = gtk_menu_item_new_with_mnemonic ("_Area");
  gtk_widget_show (zoomarea);
  gtk_container_add (GTK_CONTAINER (menu4), zoomarea);

  // Create the "Cursor" root menu, the label displayed on the menu bar
  menuitem5 = gtk_menu_item_new_with_mnemonic ("_Cursor");
  gtk_widget_show (menuitem5);
  gtk_container_add (GTK_CONTAINER (*menubar), menuitem5);

  menu5 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem5), menu5);

  curs_en_dis = gtk_menu_item_new ();
  curlabel = gtk_label_new ("Enable");
  gtk_container_add (GTK_CONTAINER (curs_en_dis), curlabel);
  gtk_widget_show (curlabel);
  gtk_widget_show (curs_en_dis);
  gtk_container_add (GTK_CONTAINER (menu5), curs_en_dis);

  // Create the "Help" root menu, the label displayed on the menu bar
  menuitem6 = gtk_menu_item_new_with_mnemonic ("_Help");
  gtk_widget_show (menuitem6);
  gtk_container_add (GTK_CONTAINER (*menubar), menuitem6);

  menu6 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem6), menu6);

  about = gtk_menu_item_new_with_mnemonic ("_About...");
  gtk_widget_show (about);
  gtk_container_add (GTK_CONTAINER (menu6), about);

  /*g_signal_connect ((gpointer) quit, "activate",
                    G_CALLBACK (gtk_main_quit),
                    NULL);*/
  g_signal_connect ((gpointer) quit, "activate",
                    G_CALLBACK (menu_quit),
                    main_window);
  g_signal_connect ((gpointer) add, "activate",
                    G_CALLBACK (trace_add_dialog),
                    NULL);
  g_signal_connect ((gpointer) remove, "activate",
                    G_CALLBACK (trace_remove_dialog),
                    NULL);
  g_signal_connect ((gpointer) xaxis, "activate",
                    G_CALLBACK (x_axis_dialog),
                    NULL);
  g_signal_connect ((gpointer) yaxis, "activate",
                    G_CALLBACK (y_axis_dialog),
                    NULL);
  g_signal_connect ((gpointer) grid, "activate",
                    G_CALLBACK (grid_dialog),
                    NULL);
  g_signal_connect ((gpointer) addplot, "activate",
                    G_CALLBACK (add_plot),
                    NULL);
  g_signal_connect ((gpointer) remplot, "activate",
                    G_CALLBACK (remove_plot),
                    NULL);
  g_signal_connect ((gpointer) zoomnormal, "activate",
                    G_CALLBACK (zoom_normal),
                    NULL);
  g_signal_connect ((gpointer) zoomarea, "activate",
                    G_CALLBACK (zoom_area),
                    NULL);
  curs_handler_id = g_signal_connect ((gpointer) curs_en_dis, "activate",
                    G_CALLBACK (cursor_enable),
                    NULL);
  g_signal_connect ((gpointer) about, "activate",
                    G_CALLBACK (about_dialog),
                    NULL);
}

void clear_to_color(GdkPixmap *pixmap, int color)
{
gint width, height;

gdk_drawable_get_size(pixmap, &width, &height);
gdk_gc_set_foreground(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], pal+color);
gdk_draw_rectangle(pixmap, drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], TRUE,
                   0, 0, width, height);
}

static gint
button_press_event (GtkWidget *widget, GdkEventButton *event,
                    EVENT *param)
{
  //g_print ("x=%d y=%d \n", (int) event->x, (int) event->y);
  //g_print ("button = %d \n", event->button);
  mouse_x = (int) event->x;
  mouse_y = (int) event->y;
  if (event->button == 1)       // left button
    {
    param->x = (int) event->x;
    param->y = (int) event->y;
    if(zoom_enabled)
      if ((param->x > curr_dataplot->rgraph.x) && (param->x < curr_dataplot->rgraph.x + curr_dataplot->rgraph.w) &&
          (param->y > curr_dataplot->rgraph.y) && (param->y < curr_dataplot->rgraph.y + curr_dataplot->rgraph.h))
        {
        param->event = STARTSELECT;
        }
    select_plot_lpress();
    }
  else if (event->button == 3)  // right button
    {
    Check_legend_click(mouse_x,mouse_y,&c2_currtrace,curr_plot);
    if (Check_plot_click(mouse_x,mouse_y,curr_plot))
        Transform_coord(mouse_x,mouse_y,&c2_screenx,&c2_screeny,curr_plot);
    Find_cur_pos(&c2_screenx,&c2_screeny,&c2_xpos,&c2_ypos,c2_currtrace);
    draw_cursor();
    }
  redraw();

  return 0;
}

static gint
button_release_event (GtkWidget *widget, GdkEventButton *event,
                      EVENT *param)
{
  double x1,y1;
  double x2,y2;

  switch (param->event)
    {
    case (STARTSELECT):
    param->event = NONE;
    ScreentoData(curr_dataplot->graphic,param->x,config.numypixels -(param->y-menu_width),&x1,&y1);
    ScreentoData(curr_dataplot->graphic,param->w,config.numypixels -(param->h-menu_width),&x2,&y2);
    set_x_limits = 1;
    plotparam.xlo = MIN(x1,x2);
    plotparam.xhi = MAX(x1,x2);
    set_y_limits = 1;
    plotparam.ylo = MIN(y1,y2);
    plotparam.yhi = MAX(y1,y2);
    Replot(curr_plot);
    redraw();

    zoom_enabled = 0;
    break;
    }

  return 0;
}

static gint
motion_notify_event (GtkWidget *widget, GdkEventButton *event,
                     EVENT *param)
{
  int x, y;
  GdkModifierType state;

  switch (param->event)
    {
    case (STARTSELECT):
    gdk_window_get_pointer (event->window, &x, &y, &state);
    param->w = (int) event->x;
    param->h = (int) event->y;

    // Check the boundaries of the plot
    if (param->w < curr_dataplot->rgraph.x)
      param->w = curr_dataplot->rgraph.x;
    if (param->w > curr_dataplot->rgraph.x + curr_dataplot->rgraph.w)
      param->w = curr_dataplot->rgraph.x + curr_dataplot->rgraph.w;
    if (param->h < curr_dataplot->rgraph.y)
      param->h = curr_dataplot->rgraph.y;
    if (param->h > curr_dataplot->rgraph.y + curr_dataplot->rgraph.h)
      param->h = curr_dataplot->rgraph.y + curr_dataplot->rgraph.h;

    blit(bmp,bmptmp,0,0,0,0,config.numxpixels,config.numypixels);
    gdk_gc_set_foreground(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], pal+5);
    gdk_draw_rectangle(bmptmp,main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                       FALSE,
                       MIN(param->x, param->w),
                       MIN(param->y, param->h),
//                       (param->x < param->w) ? param->x : param->w,
//                       (param->y < param->h) ? param->y : param->h,
                       abs(param->w - param->x), abs(param->h - param->y));
    blit(bmptmp,drawing_area->window,0,0,0,0,config.numxpixels,config.numypixels);
    break;
    }

  return 0;
}

int
MFBInitialize( )
{
GtkStyle *style;
GdkBitmap *icon_mask = NULL;
GdkPixmap *icon_pm = NULL;

if (graph_mode_on == 0)
    {
    // Override background and foreground (text) colors
    MFBDefineColor(0, 0, 0, 0);        // background = black
    MFBDefineColor(1, 0, 255, 255);    // foreground = cyan

    // Define new colors
    MFBDefineColor(20, 255, 255, 255); // cursor = white
    MFBDefineColor(21, 0, 0, 156);     // grid lines = dark blue

    // Override line styles
    /*line_styles[1] = 0x1010;  // grid subdivision lines
    line_styles[2] = 0xaaaa;  // grid decade lines
    line_styles[3] = 0xaaaa;  // first cursor
    line_styles[4] = 0x4444;  // second cursor*/

    //gtk_init(&ac, &av);
    gtk_init(NULL, NULL);

    // Create a new window and set default size
    /*if (main_window != NULL)
        gtk_widget_unref(main_window);*/
    main_window = NULL;
    drawing_area = NULL;
    main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_default_size (GTK_WINDOW (main_window), mfbpc_width, mfbpc_height + 26);

    // Create a VBox
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (main_window), vbox);
    gtk_widget_show (vbox);

    // Menu
    //get_main_menu (main_window, &menubar);
    get_main_menu2 (main_window, &menubar);
    gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, TRUE, 0);
    gtk_widget_show (menubar);

    // Set palette color
    set_pal();

    // Drawing area
    drawing_area = gtk_drawing_area_new ();
    //gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), mfbpc_width, mfbpc_height);
    gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
    gtk_widget_show (drawing_area);

    // Signals connection
    gtk_signal_connect (GTK_OBJECT (main_window), "delete_event",
                       GTK_SIGNAL_FUNC (on_delete_event), NULL);
    gtk_signal_connect (GTK_OBJECT (main_window), "destroy",
                       GTK_SIGNAL_FUNC (on_destroy), NULL);

    gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
                        (GtkSignalFunc) on_expose_event, NULL);
    gtk_signal_connect (GTK_OBJECT(drawing_area),"configure_event",
                        (GtkSignalFunc) on_configure_event, NULL);

    g_signal_connect (G_OBJECT (drawing_area), "button_press_event",
                      G_CALLBACK (button_press_event), &main_event);
    g_signal_connect (G_OBJECT (drawing_area), "button_release_event",
                      G_CALLBACK (button_release_event), &main_event);
    g_signal_connect (G_OBJECT (drawing_area), "motion_notify_event",
                      G_CALLBACK (motion_notify_event), &main_event);

    gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK
                       | GDK_LEAVE_NOTIFY_MASK
                       | GDK_BUTTON_PRESS_MASK
                       | GDK_POINTER_MOTION_MASK
                       | GDK_POINTER_MOTION_HINT_MASK
                       | GDK_BUTTON_RELEASE_MASK);
    gtk_widget_set_extension_events (drawing_area, GDK_EXTENSION_EVENTS_CURSOR);

    // Show the window
    num_plot = 1;
    gtk_window_set_position(GTK_WINDOW(main_window), GTK_WIN_POS_CENTER);
    createwindow = 1;
    gtk_widget_show(main_window);
    createwindow = 0;
    gdk_window_set_title(main_window->window, TitleName);

    style = gtk_widget_get_style(main_window);
    icon_pm = gdk_pixmap_create_from_xpm_d(main_window->window, &icon_mask, &style->bg [GTK_STATE_NORMAL], (gchar **) spicep_ico_xpm);
    gdk_window_set_icon(main_window->window, NULL, icon_pm, icon_mask);

    graph_mode_on = 1;
    }
else
    {
    clear_to_color(currbmp,0);
    }

#ifdef WANT_PCHARDCOPY
    _dos_setvect(5, mfbpc_hardcopy);
#endif
    return (MFBOK);
}

void
MFBClearScreen( )
{
    /*_clearscreen(_GCLEARSCREEN);
    _settextposition(1, 1);*/
}

void
MFBHome( )
{
    //_settextposition(1, 1);
}

void
MFBClearEol( )
{
    printf("            ");
}

int
MFBSetColor(color)
	int	color;
{

    if (color > 0) {
	color = color % (config.numcolors - 1);
    currcolor = color;
	//_setcolor((short) (config.numcolors - color));
    } else
	//_setcolor(0);
    currcolor = 0;

    //gdk_rgb_find_color(colormap, pal+currcolor);
    gdk_gc_set_foreground(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], pal+currcolor);

    return (MFBOK);
}

/* Don't ever want to do this */

int
MFBFlood()
{
    return (MFBOK);
}

int
MFBSetTextMode(dest)
{
    return (MFBOK);
}

int
MFBSetALUMode(mode)
{
    return (MFBOK);
}

gint8 *mask2dashlist(int mask, int *num_dash_list)
{
gint8 *dashlist;
int tmpmask;
int refbit, currbit, firstbit;
int numvar;
int i;
int odd = 0;
int count;
int nshift;

// Find the number of bit variations in the dash_list
tmpmask = mask;
refbit = tmpmask & 0x0001;
tmpmask = tmpmask >> 1;
currbit = tmpmask & 0x0001;
numvar = 0;
for (nshift=1; nshift<=15; nshift++)
  {
  if (currbit != refbit)
    {
    numvar++;
    refbit = currbit;
    }
  tmpmask = tmpmask >> 1;
  currbit = tmpmask & 0x0001;
  }

// Find the number of elements in the dash_list
*num_dash_list = numvar + 1;
firstbit = (mask >> 15) & 0x0001;
if (!firstbit)
  (*num_dash_list)++;
if ((*num_dash_list) % 2 != 0)
  {
  odd = 1;
  (*num_dash_list)++;
  }

// Allocation of the dashlist
dashlist = (gint8 *) malloc ((*num_dash_list) * sizeof(int));

i=(*num_dash_list)-1;
if (odd)
  {
  dashlist[i] = 0;  // pad with zero
  i--;
  }

tmpmask = mask;
refbit = tmpmask & 0x0001;
for (nshift=0; nshift<16; nshift++)
  {
  count=1;
  tmpmask = tmpmask >> 1;
  currbit = tmpmask & 0x0001;
  while ((currbit == refbit) && (nshift<15))
    {
    count++;
    nshift++;
    tmpmask = tmpmask >> 1;
    currbit = tmpmask & 0x0001;
    }
  refbit = currbit;
  dashlist[i] = count;
  i--;
  }

if (firstbit == 0)
  dashlist[0] = 0;

return dashlist;
}

gint8 *mask2dashlist_2(int mask, int *num_dash_list, int *offset)
{
gint8 *dashlist;
int tmpmask;
int refbit, currbit, firstbit;
int numvar;
int i;
int count;
int nshift;

// Rotate left mask until firstbit is equal to 1
firstbit = (mask >> 15) & 0x0001;
*offset = 0;
while (firstbit == 0)
  {
  mask = mask << 1;
  firstbit = (mask >> 15) & 0x0001;
  (*offset)++;
  }

// Find the number of bit variations in the dash_list
tmpmask = mask;
refbit = tmpmask & 0x0001;
tmpmask = tmpmask >> 1;
currbit = tmpmask & 0x0001;
numvar = 0;
for (nshift=1; nshift<=15; nshift++)
  {
  if (currbit != refbit)
    {
    numvar++;
    refbit = currbit;
    }
  tmpmask = tmpmask >> 1;
  currbit = tmpmask & 0x0001;
  }

// Find the number of elements in the dash_list
*num_dash_list = numvar + 1;
firstbit = (mask >> 15) & 0x0001;

// Allocation of the dashlist
dashlist = (gint8 *) malloc ((*num_dash_list) * sizeof(int));

i=(*num_dash_list)-1;

tmpmask = mask;
refbit = tmpmask & 0x0001;
for (nshift=0; nshift<16; nshift++)
  {
  count=1;
  tmpmask = tmpmask >> 1;
  currbit = tmpmask & 0x0001;
  while ((currbit == refbit) && (nshift<15))
    {
    count++;
    nshift++;
    tmpmask = tmpmask >> 1;
    currbit = tmpmask & 0x0001;
    }
  refbit = currbit;
  dashlist[i] = count;
  i--;
  }

return dashlist;
}

int
MFBSetLineStyle(linestyleid)
    int linestyleid;
{
    gint8 *dash_list;
    //gint8 dash_list[16];
    int num_dash_list;
    int offset;

    sel_style = line_styles[linestyleid % NSTYLES];
    //_setlinestyle(line_styles[style % NSTYLES]);

    if (sel_style == 0xffff)
      gdk_gc_set_line_attributes(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
                                 1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
    else
      {
      //dash_list = mask2dashlist(sel_style, &num_dash_list);
      dash_list = mask2dashlist_2(sel_style, &num_dash_list, &offset);
      gdk_gc_set_line_attributes(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
                                 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER);
      gdk_gc_set_dashes(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], 0, dash_list, num_dash_list);
      free(dash_list);
      }

    return (MFBOK);
}

int
MFBLine(fromx, fromy, tox, toy)
	int fromx, fromy;
	int tox, toy;
{
    //_moveto((short) fromx, (short) (config.numypixels - fromy));
    //_lineto((short) tox, (short) (config.numypixels - toy));
    gdk_draw_line(currbmp,main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                  fromx, config.numypixels - fromy, tox, config.numypixels - toy);
    return (MFBOK);
}

int
MFBHLine(fromx, fromy, tox)
    int fromx, fromy;
    int tox;
{
    /*gint8 *dash_list;
    int num_dash_list;

    dash_list = mask2dashlist(sel_style, &num_dash_list);
    gdk_gc_set_line_attributes(main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                           1, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER);
    gdk_gc_set_dashes(main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)], 0, dash_list, num_dash_list);
    free(dash_list);*/

    gdk_draw_line(currbmp,main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                  fromx, config.numypixels - fromy, tox, config.numypixels - fromy);
    /*gdk_gc_set_fill(main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)], GDK_SOLID);
    gdk_gc_set_line_attributes(main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                           1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);*/

    return (MFBOK);
}

int
MFBVLine(fromx, fromy, toy)
    int fromx, fromy;
    int toy;
{
    /*gint8 *dash_list;
    int num_dash_list;

    dash_list = mask2dashlist(sel_style, &num_dash_list);
    gdk_gc_set_line_attributes(main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                           1, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER);
    gdk_gc_set_dashes(main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)], 0, dash_list, num_dash_list);
    free(dash_list);*/

    gdk_draw_line(currbmp,main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                  fromx, config.numypixels - fromy, fromx, config.numypixels - toy);
    /*gdk_gc_set_fill(main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)], GDK_SOLID);
    gdk_gc_set_line_attributes(main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                           1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);*/

    return (MFBOK);
}

int
MFBRect(fromx, fromy, tox, toy)
    int fromx, fromy;
    int tox, toy;
{
    //Rectangle(currbmp, fromx, config.numypixels - fromy, tox, config.numypixels - toy);
    return (MFBOK);
}

int
MFBText(text, x, y, degrees)
    char *text;
    int x, y, degrees;
{
    char	c, *s;
    short	x0, y0;
    int		i, j;
    unsigned char	bits, p;

    y = (config.numypixels - y) - 8;
    for (s = text; s && *s; s++) {
	//if (*s > 127)
	//   	*s -= 128;
	c = *s - 33;
	if (c >= 0 && c < 96) {
	    y0 = y;
	    for (i = 0; i < 7; i++) {
		x0 = x;
		p = 0x80;
		bits = c_map[(int)c][i];
		for (j = 0; j < 7; j++) {
		    if (p & bits)
            gdk_draw_point(bmp,main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],x0,y0);
			//_setpixel(x0, y0);
		    p = p >> 1;
		    x0 += 1;
		}
		y0 += 1;
	    }
	}
	if (!degrees)
	    x += 7;
	else
	    y += 8;
    }

#ifdef notdef
    short	tx, ty;

    if (!text)
	return MFBOK;

    tx = x * config.numtextcols / config.numxpixels;
    ty = (config.numypixels - y) * config.numtextrows / config.numypixels;
    _settextposition(ty, tx);
    _outtext(text);

    printf("\"%s\" @ %d,%d // %d\n", text, x, y, degrees);
    if (!degrees)
	    _outtext(text);
    else {
	static char lb[ ] = "x\n\0";
	struct rccoord curp;
	for (s = text; *s; s++) {
	    lb[0] = *s;
	    _outtext(lb);
	    curp = _gettextposition( );
	    _settextposition((short) x, (short) curp.col);
	}
    }
#endif

    return (MFBOK);
}

int
MFBDrawLineTo(x, y)
	int x, y;
{
    //_lineto((short) x, (short) (config.numypixels - y));
    return (MFBOK);
}

int
MFBUpdate()
{
    /* Do nothing */
    return (MFBOK);
}

int
MFBBox(left, bottom, right, top)
{
    /* We don't use this in nutmeg, really. */
    return (MFBOK);
}

int
MFBDefineColor(num, red, green, blue)
{
    /* No-op. */
    pal[num].red   = red*65535/255;
    pal[num].green = green*65535/255;
    pal[num].blue  = blue*65535/255;
    return (MFBOK);
}

static int styles[] = { 0333, 0347, 0360, 0071, 0216, 0366, 0155, 0011,
            0123, 0317, 0330, 0145, 0275, 0016, 0315, 0101,
            0222, 0174, 0251, 0376 } ;

int
MFBDefineLineStyle(num, mask)
{

    if ((num < 0) || (num >= NSTYLES))
        return (MFBBADLST);
    styles[num] = mask;

    return (MFBOK);
}

int
MFBClose()
{
    /* Reset the hardcopy interrupt vector */
#ifdef WANT_PCHARDCOPY
    _dos_setvect(5, hc_vect);
#endif

    /* Reset the graphics mode */
    //_setvideomode(_DEFAULTMODE);

#ifdef notdef
    printf("x = %d, y = %d, colors = %d, def = %d\n",
	(int) config.numxpixels, (int) config.numypixels,
	(int) config.numcolors, (int) graphics_mode);
    printf("cx = %d, cy = %d\n", config.numtextrows, config.numtextcols);
#endif

    return (MFBOK);
}

int
MFBHalt()
{
    return (MFBClose());
}

/* Print a MFB error string... */

static struct errinfo {
    int er_num;
    char *er_string;
} errstrings[] = {
    { MFBOK,        "" } ,
    { MFBBADENT,        "Unknown terminal type" } ,
    { MFBBADMCE,        "Bad mfbcap entry" } ,
    { MFBBADLST,        "Illegal line style" } ,
    { MFBBADFST,        "Illegal fill style" } ,
    { MFBBADCST,        "Illegal color style" } ,
    { MFBBADTM1,        "No destructive text mode" } ,
    { MFBBADTM2,        "No overstrike text mode" } ,
    { MFBNODFLP,        "No definable line styles" } ,
    { MFBNODFFP,        "No definable fill styles" } ,
    { MFBNODFCO,        "No definable colors" } ,
    { MFBNOBLNK,        "No blinkers" } ,
    { MFBBADDEV,        "Can't open or close device" } ,
    { MFBBADOPT,        "Can't access or set device status" } ,
    { MFBNOMASK,        "No definable read or write mask" } ,
    { MFBBADWRT,        "Error in write" } ,
    { MFBNOPNT,     "No pointing device" } ,
    { MFBNOMEM,     "Out of memory" } ,
    { 0,            NULL }
} ;

char *
MFBError(num)
{
    struct errinfo *ei;
    static char buf[32];

    for (ei = errstrings; ei->er_string; ei++)
        if (ei->er_num == num)
            break;
    if (ei->er_string)
        return (ei->er_string);
    else {
        sprintf(buf, "Unknown error # %d", num);
        return (buf);
    }
}

/* The mfb struct will not contain anything valid. */

int
MFBInfo(num)
{
    int	result;
    int goback = 0;

    if (!initialized) {
	MFBOpen("pc",NULL,NULL);
	goback = 1;
    }

    switch (num) {
        case MAXX:
            result = config.numxpixels;
	    break;
        case MAXY:
            result = config.numypixels;
	    break;
        case MAXCOLORS:
            result = config.numcolors - 1;
	    break;
        case MAXLINESTYLES:
            result = NSTYLES;
	    break;
        case TEXTROTATABLE:
            result = 0; /* It can, but it's too ugly now... */
	    break;
        case FONTHEIGHT:
	    result = 8;
	    break;
        case FONTWIDTH:
	    result = 7;
	    break;
        case DEFLINEPATTERN:
            result = 1;
	    break;
        case DESTRUCTIVETEXT:
            result = 1;
	    break;
        case OVERSTRIKETEXT:
            result = 0;
	    break;

        default:
            result = -1;
    }

    if (goback)
	MFBClose( );

    return result;
}

/*ARGSUSED*/
void
MFBArc(x,y,r,theta1,theta2,s)
    int x;      /* x coordinate of center */
    int y;      /* y coordinate of center */
    int r;      /* radius of arc */
    double theta1;       /* initial angle ( +x axis = 0 degrees ) */
    double theta2;  /* final angle ( +x axis = 0 degrees ) */
    int s;      /* number of segments in a 360 degree arc */
    /*
     * Notes:
     *    Draws an arc of radius r and center at (x,y) beginning at
     *    angle astart (in degrees) and ending at astop
     */
{
    short	left, right, top, bottom;
    short	x0, y0, x1, y1;
    gint        angle1, angle2;

    //y = config.numypixels - y;

    left = x - r;
    right = x + r;
    top = y + r;
    bottom = y - r;

    x0 = x + r * cos(theta1);
    y0 = y + r * cos(theta1);
    x1 = x + r * cos(theta2);
    y1 = y + r * cos(theta2);

    angle1 = (gint) (theta1 * 180 * 64 / M_PI );
    angle2 = (gint) ((theta2 - theta1) * 180 * 64 / M_PI );
    if (angle2 == 0)
        angle2 = 23040;

    //_arc(left, top, right, bottom, x0, y0, x1, y1);
    gdk_draw_arc(currbmp, main_window->style->fg_gc[GTK_WIDGET_STATE (main_window)],
                 FALSE, left, config.numypixels-top, 2*r, 2*r, angle1, angle2);
}

/*void (interrupt far mfbpc_hardcopy)( )
{
    static char	*string;
    static int	count;
    static int	i, j, k, l;
    static char	*buf;
    static int	groupwidth;
    static unsigned short a;
    static union REGS in;

    print("\r\n\0333\030", 5);

    mfbpc_buffer[0] = '\r';
    mfbpc_buffer[1] = '\n';
    mfbpc_buffer[2] = '\033';
    mfbpc_buffer[3] = '\K';
    if (!mfbpc_double) {
	mfbpc_buffer[4] = mfbpc_height % 256;
	mfbpc_buffer[5] = mfbpc_height / 256;
    } else {
	mfbpc_buffer[4] = (2 * mfbpc_height) % 256;
	mfbpc_buffer[5] = (2 * mfbpc_height) / 256;
    }

    buf = mfbpc_buffer + 6;

    for (i = 0; i < mfbpc_width; i += 8) {
	groupwidth = mfbpc_width - i;
	if (groupwidth > 8)
	    groupwidth = 8;
	if (mfbpc_double) {
            for (j = 0; j < mfbpc_height + mfbpc_height; j += 1)
	        buf[j] = 0;
	} else {
            for (j = 0; j < mfbpc_height; j += 1)
	        buf[j] = 0;
	}
	for (j = 0; j < groupwidth; j += 1) {
	    l = 0;
	    for (k = 0; k < mfbpc_height; k += 1) {
		a = _getpixel((short) mfbpc_width - i - j - 1, (short) k);
		if (a)
		    buf[l] |= 0x80 >> j;
		if (mfbpc_double) {
		    l += 1;
		    if (a) {
			buf[l] |= 0x80 >> j;
		    }
		}
		l += 1;
	    }
	}

	if (!mfbpc_double) {
	    print(mfbpc_buffer, mfbpc_height + 6);
	} else {
	    print(mfbpc_buffer, mfbpc_height + mfbpc_height + 6);
	}
    }

    print(FINI_PRINTER, sizeof(FINI_PRINTER) - 1);

}*/
