/**********
Copyright (c) 2003-2004 Paolo Fallucca.
**********/

#include "spice.h"

#include "ftedev.h"
#include "ftedefs.h"
#include "ftegraph.h"
#include "ftedbgra.h"
#include "cpdefs.h"
#include "mfbpc.h"
#include <locale.h>

#ifdef ALLEGRO
#include <allegro.h>

extern void redraw();
extern MENU cursor_menu[];
extern DIALOG the_dialog[];
extern void zoom_init(int dlgnum);
#endif

#ifdef GDI
#define redraw() InvalidateRect(hwnd, NULL, false)

#define blit(srcbmp,destbmp,src_x,src_y,dest_x,dest_y,width,height) \
        BitBlt(destbmp,dest_x,dest_y,width,height,srcbmp,src_x,src_y,SRCCOPY)
extern void draw_cursor();
#endif

#ifdef GTK
//#define redraw() gdk_window_invalidate_rect(drawing_area->window, NULL, TRUE)
#define redraw() gdk_gc_set_foreground(drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], pal); \
                 gdk_window_invalidate_rect(drawing_area->window, NULL, TRUE)
#define blit(srcbmp,destbmp,src_x,src_y,dest_x,dest_y,width,height) \
        gdk_draw_pixmap(destbmp,drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],srcbmp,src_x,src_y,dest_x,dest_y,width,height)
extern void draw_cursor();
extern GdkColor pal[256];
#endif

extern int cursor_enabled;
int set_x_limits = 0;
int set_y_limits = 0;
int set_xaxis = 0;

wordlist *
vec2wl_copy(struct dvec *vecs)
{
    register wordlist *nwl = NULL, *w = NULL;
    register struct dvec *dv;

    for (dv = vecs; dv; dv = dv->v_link2) {
        if (nwl == NULL) {
            nwl = w = alloc(struct wordlist);
	    w->wl_prev = NULL;
	    w->wl_next = NULL;
        } else {
            w->wl_next = alloc(struct wordlist);
            w->wl_next->wl_prev = w;
            w = w->wl_next;
	    w->wl_next = NULL;
        }
        w->wl_word = copy(dv->v_name);
    }
    return (nwl);
}

void free_tracelist(wordlist **wlist)
{
wordlist **wl, *nw;

for (wl = wlist; *wl; *wl = nw)
    {
    nw = (*wl)->wl_next;
    tfree((*wl)->wl_word);
    tfree(*wl);
    }
}

int add_plot()
{
int i;
//int xoff;

clear_to_color(plotsbmp,0);

config.numypixels = (mfbpc_height - menu_width) / (num_plot + 1);
dispdev->height = config.numypixels + 1;

for (i = 0; i < num_plot; i++)
    //Replot(i,&xoff);
    Replot(i);
    
// Plot an empty grid
clear_to_color(bmp,0);
gr_redrawgrid(currentgraph);
blit(bmp,plotsbmp,0,0,0,config.numypixels * num_plot,
     config.numxpixels,config.numypixels);

num_plot++;
curr_plot = num_plot - 1;
#ifdef ALLEGRO
cursor_menu[0].flags = D_DISABLED;
the_dialog[CURSOR_POS].flags = D_HIDDEN;
#endif

// Set the new plot rectangular region with the values
// of the previous plot
dataplot[curr_plot].rgraph.x = dataplot[curr_plot - 1].rgraph.x;
dataplot[curr_plot].rgraph.y = dataplot[curr_plot - 1].rgraph.y;
dataplot[curr_plot].rgraph.w = dataplot[curr_plot - 1].rgraph.w;
dataplot[curr_plot].rgraph.h = dataplot[curr_plot - 1].rgraph.h;

#ifdef ALLEGRO
// Setting the mouse clickable region for plot select
//the_dialog[SEL_PLOT].proc(MSG_USER,&the_dialog[SEL_PLOT],0);
the_dialog[SEL_PLOT].x = 0;
the_dialog[SEL_PLOT].y = menu_width;
the_dialog[SEL_PLOT].w = mfbpc_width;
the_dialog[SEL_PLOT].h = mfbpc_height - menu_width;
#endif

curr_dataplot = &dataplot[curr_plot];
redraw();
return 0;
}

int remove_plot()
{
int i;

clear_to_color(plotsbmp,0);
config.numypixels = (mfbpc_height - menu_width) / (num_plot - 1);
dispdev->height = config.numypixels + 1;

// re-copy all the tracelists following the removed one
for (i = curr_plot + 1; i < num_plot; i++)
    {
    dataplot[i-1].tracelist = wl_copy(dataplot[i].tracelist);
    }
// and free the last tracelist
free_tracelist(&(dataplot[num_plot-1].tracelist));
curr_plot = 0;
num_plot--;

// replot all
for (i = 0 ;i < num_plot ; i++)
    Replot(i);
redraw();

curr_dataplot = &dataplot[0];
return 0;
}

void select_plot_lpress()
{
int i;
int old_plot;

old_plot = curr_plot;
for (i = 0; i<num_plot; i++)
    if(Check_plot_click(mouse_x,mouse_y,i))
        {
        curr_plot = i;
        curr_dataplot = &dataplot[i];
        }
//blit(plotsbmp,bmp,0,config.numypixels * curr_plot,0,0,
//     config.numxpixels,config.numypixels);
redraw();

#ifdef ALLEGRO
if (curr_dataplot->graphic)
    cursor_menu[0].flags = 0;

if ((!(strcmp(cursor_menu[0].text,"&Disable"))) &&
    (the_dialog[CURSOR_POS].flags == D_HIDDEN))
    if (curr_dataplot->graphic)
        the_dialog[CURSOR_POS].flags = 0;
#endif

// Cursor handler
if (cursor_enabled)
    {
    if (old_plot == curr_plot)  // click on the same plot
        {
        Check_legend_click(mouse_x,mouse_y,&c1_currtrace,curr_plot);
        if (Check_plot_click(mouse_x,mouse_y,curr_plot))
            Transform_coord(mouse_x,mouse_y,&c1_screenx,&c1_screeny,curr_plot);
        Find_cur_pos(&c1_screenx,&c1_screeny,&c1_xpos,&c1_ypos,c1_currtrace);
        #ifdef ALLEGRO
        the_dialog[CURSOR_POS].proc(MSG_DRAW,&the_dialog[CURSOR_POS],0);
        #else
        draw_cursor();
        #endif
        }
    else  // click on different plot
        {
        // Disable old cursor
        blit(bmp,plotsbmp,0,0,0,config.numypixels * old_plot,
            config.numxpixels,config.numypixels);
        redraw();

        #ifdef ALLEGRO
        cursor_init();
        #else
        get_cursor_init_pos();
        draw_cursor();
        #endif
        }
    }

#ifdef ALLEGRO
if (the_dialog[ZOOM_AREA].flags == 0) // zoom enabled
    {
    if (Check_plot_click(mouse_x,mouse_y,curr_plot))
        {
        zoom_init(ZOOM_AREA);
        the_dialog[ZOOM_AREA].proc(MSG_START,&the_dialog[ZOOM_AREA],0);
        the_dialog[ZOOM_AREA].proc(MSG_LPRESS,&the_dialog[ZOOM_AREA],0);
        }
    }
#endif

set_plot_param();
check_param();
}

void set_plot_param()
{
// Set plotparam with the values of currentgraph
if (curr_dataplot->graphic)
    {
    plotparam.gtype = (curr_dataplot->graphic)->grid.gridtype;
    plotparam.xlo = (curr_dataplot->graphic)->datawindow.xmin;
    plotparam.xhi = (curr_dataplot->graphic)->datawindow.xmax;
    plotparam.ylo = (curr_dataplot->graphic)->datawindow.ymin;
    plotparam.yhi = (curr_dataplot->graphic)->datawindow.ymax;
    plotparam.xaxis = copy(*plot_cur->pl_scale);
    }
}

void Replot(int plot)
{
wordlist *wparlist, *wlist;
char string[DIMSTRING];
char *old_locale, *saved_locale;

curr_dataplot = &dataplot[plot];
wlist = wl_copy(curr_dataplot->tracelist);

// Transform parameters in command line
// Grid type
switch (plotparam.gtype)
    {
    case GRID_POLAR:
    wl_add(&wlist,"polar");
    break;

    case GRID_SMITH:
    wl_add(&wlist,"smith");
    break;

    case GRID_SMITHGRID:
    wl_add(&wlist,"smithgrid");
    break;

    case GRID_XLOG:
    wl_add(&wlist,"xlog");
    break;

    case GRID_YLOG:
    wl_add(&wlist,"ylog");
    break;

    case GRID_LOGLOG:
    wl_add(&wlist,"loglog");
    break;

    case GRID_LIN:
    wl_add(&wlist,"linear");
    break;

    case GRID_NONE:
    break;
    }

old_locale = setlocale(LC_ALL, NULL);
saved_locale = (char *) strdup(old_locale);
setlocale(LC_ALL, "C");

// Set x and y limits
if (set_x_limits)
    {
    sprintf(string,"xl %g %g",plotparam.xlo,plotparam.xhi);
    wparlist = cp_parse(string);
    wlist = wl_append(wlist,wparlist);
    }
if (set_y_limits)
    {
    sprintf(string,"yl %g %g",plotparam.ylo,plotparam.yhi);
    wparlist = cp_parse(string);
    wlist = wl_append(wlist,wparlist);
    }

// Set x axis
if (set_xaxis)
    {
    sprintf(string,"vs %s",plotparam.xaxis);
    wparlist = cp_parse(string);
    wlist = wl_append(wlist,wparlist);
    }

setlocale(LC_ALL, saved_locale);
    
com_plot(wl_copy(wlist));

// After plot command currentgraph change, so you have to re-copy it
curr_dataplot->graphic = CopyGraph(currentgraph);
// and re-calculate graph region
Set_plot_rect(&(curr_dataplot->rgraph),curr_dataplot->graphic);
// Set plotparam with the values of currentgraph
set_plot_param();
check_param();

blit(bmp,plotsbmp,0,0,0,config.numypixels * plot,
     config.numxpixels,config.numypixels);
}

